@@ -1,41 +1,12 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #define GLX_GLXEXT_LEGACY
 #define GL_GLEXT_PROTOTYPES
 #define _GNU_SOURCE
-#include <GL/glx.h>
-#include <GL/gl.h>
-#include <GL/glext.h>
 #include <dlfcn.h>
-#include <link.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -55,41 +26,74 @@
 #include <math.h>
 #include <errno.h>
 #include <time.h>
+#include <limits.h>
+
+#if defined(TARGET_UNIX)
+# define GLX_GLXEXT_LEGACY
+# define GL_GLEXT_PROTOTYPES
+# define _GNU_SOURCE
+# include <GL/glx.h>
+# include <GL/gl.h>
+# include <GL/glext.h>
+
+#include <link.h>
 
 typedef unsigned char bool;
-#define true 1
-#define false 0
+# define true 1
+# define false 0
+#elif defined(TARGET_MAC)
+# include <OpenGL/OpenGL.h>
+# include <Carbon/Carbon.h>
+# include <Cocoa/Cocoa.h>
+# include <AGL/agl.h>
+#
+# include <objc/objc-runtime.h>
+#
+# include "mach_override.h"
+#
+# include "avail_mac.h"
+#endif
 
 #include "../overlay/overlay.h"
 
-// Prototypes
-static void ods(const char *format, ...);
-
-static bool bDebug;
+static bool bDebug = false;
+static bool bCursorAvail = false;
 
 typedef struct _Context {
 	struct _Context *next;
+
+#if defined(TARGET_UNIX)
 	Display *dpy;
 	GLXDrawable draw;
+#elif defined(TARGET_MAC)
+	CGLContextObj cglctx;
+	NSOpenGLContext *nsctx;
+#endif
 
 	unsigned int uiWidth, uiHeight;
 	unsigned int uiLeft, uiRight, uiTop, uiBottom;
 
 	struct sockaddr_un saName;
 	int iSocket;
+	// overlay message, temporary variable for processing from socket
 	struct OverlayMsg omMsg;
+	// opengl overlay texture
 	GLuint texture;
 
+	// overlay texture in shared memory
 	unsigned char *a_ucTexture;
 	unsigned int uiMappedLength;
 
 	bool bValid;
-	bool bGlx;
+	bool bMesa;
 
 	GLuint uiProgram;
 
 	clock_t timeT;
 	unsigned int frameCount;
+
+	GLint maxVertexAttribs;
+	GLboolean* vertexAttribStates;
 } Context;
 
 static const char vshader[] = ""
@@ -101,42 +105,30 @@ static const char vshader[] = ""
 static const char fshader[] = ""
                               "uniform sampler2D tex;"
                               "void main() {"
-			      "gl_FragColor = texture2D(tex, gl_TexCoord[0].st);"
+                              "gl_FragColor = texture2D(tex, gl_TexCoord[0].st);"
                               "}";
 
 const GLfloat fBorder[] = {0.125f, 0.250f, 0.5f, 0.75f};
 
 static Context *contexts = NULL;
 
+#define AVAIL(name) dlsym(RTLD_DEFAULT,#name)
 #define FDEF(name) static __typeof__(&name) o##name = NULL
 
-FDEF(dlsym);
-
-FDEF(glXSwapBuffers);
-FDEF(glXGetProcAddressARB);
-FDEF(glXGetProcAddress);
-
-#define RESOLVE(x) if (! o##x) o##x = (__typeof__(&x)) odlsym(RTLD_NEXT, #x)
-
-static void resolveOpenGL() {
-	RESOLVE(glXSwapBuffers);
-
-	if (! oglXSwapBuffers) {
-		void *lib = dlopen("libGL.so.1", RTLD_GLOBAL | RTLD_NOLOAD);
-		if (! lib)
-			return;
-		RESOLVE(glXSwapBuffers);
-		if (! oglXSwapBuffers)
-			dlclose(lib);
-	}
-
-	RESOLVE(glXGetProcAddressARB);
-	RESOLVE(glXGetProcAddress);
-}
+#if defined(TARGET_UNIX)
+ FDEF(dlsym);
+ FDEF(glXSwapBuffers);
+ FDEF(glXGetProcAddressARB);
+ FDEF(glXGetProcAddress);
+#elif defined(TARGET_MAC)
+ FDEF(CGLFlushDrawable);
+ FDEF(CGDisplayHideCursor);
+ FDEF(CGDisplayShowCursor);
+#endif
 
 __attribute__((format(printf, 1, 2)))
 static void ods(const char *format, ...) {
-	if (!bDebug) {
+	if (! bDebug) {
 		return;
 	}
 
@@ -153,18 +145,25 @@ static void ods(const char *format, ...) {
 static void newContext(Context * ctx) {
 	ctx->iSocket = -1;
 	ctx->omMsg.omh.iLength = -1;
-	ctx->texture = ~0;
+	ctx->texture = ~0U;
 	ctx->timeT = clock();
 	ctx->frameCount = 0;
 
 	char *home = getenv("HOME");
 	if (home == NULL) {
 		struct passwd *pwent= getpwuid(getuid());
-		if (pwent && pwent->pw_dir && pwent->pw_dir[0])
+		if (pwent && pwent->pw_dir && pwent->pw_dir[0]) {
 			home = pwent->pw_dir;
+		}
 	}
 
-	if (home) {
+	char *xdgRuntimeDir = getenv("XDG_RUNTIME_DIR");
+
+	if (xdgRuntimeDir != NULL) {
+		ctx->saName.sun_family = PF_UNIX;
+		strcpy(ctx->saName.sun_path, xdgRuntimeDir);
+		strcat(ctx->saName.sun_path, "/MumbleOverlayPipe");
+	} else if (home) {
 		ctx->saName.sun_family = PF_UNIX;
 		strcpy(ctx->saName.sun_path, home);
 		strcat(ctx->saName.sun_path, "/.MumbleOverlayPipe");
@@ -190,6 +189,9 @@ static void newContext(Context * ctx) {
 	glAttachShader(ctx->uiProgram, vs);
 	glAttachShader(ctx->uiProgram, fs);
 	glLinkProgram(ctx->uiProgram);
+
+	glGetIntegerv(GL_MAX_VERTEX_ATTRIBS, &ctx->maxVertexAttribs);
+	ctx->vertexAttribStates = calloc((size_t) ctx->maxVertexAttribs, sizeof(GLboolean));
 }
 
 static void releaseMem(Context *ctx) {
@@ -200,7 +202,7 @@ static void releaseMem(Context *ctx) {
 	}
 	if (ctx->texture != ~0U) {
 		glDeleteTextures(1, &ctx->texture);
-		ctx->texture = ~0;
+		ctx->texture = ~0U;
 	}
 	ctx->uiLeft = ctx->uiTop = ctx->uiRight = ctx->uiBottom = 0;
 }
@@ -217,19 +219,21 @@ static void disconnect(Context *ctx) {
 
 static bool sendMessage(Context *ctx, struct OverlayMsg *om) {
 	if (ctx->iSocket != -1) {
-		ssize_t wantsend = sizeof(struct OverlayMsgHeader) + om->omh.iLength;
+		size_t wantsend = sizeof(struct OverlayMsgHeader) + (size_t)om->omh.iLength;
 		ssize_t sent = send(ctx->iSocket, om, wantsend, MSG_DONTWAIT);
-		if (wantsend == sent)
+		if (sent != -1 && wantsend == (size_t)sent) {
 			return true;
-		ods("Short write");
+		}
+		ods("Short write. Disconnecting pipe.");
 	}
 	disconnect(ctx);
 	return false;
 }
 
 static void regenTexture(Context *ctx) {
-	if (ctx->texture != ~0U)
+	if (ctx->texture != ~0U) {
 		glDeleteTextures(1, & ctx->texture);
+	}
 	glGenTextures(1, &ctx->texture);
 
 	glBindTexture(GL_TEXTURE_2D, ctx->texture);
@@ -239,10 +243,11 @@ static void regenTexture(Context *ctx) {
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, ctx->uiWidth, ctx->uiHeight, 0, GL_BGRA, GL_UNSIGNED_BYTE, ctx->a_ucTexture);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, (GLsizei)ctx->uiWidth, (GLsizei)ctx->uiHeight, 0, GL_BGRA, GL_UNSIGNED_BYTE, ctx->a_ucTexture);
 }
 
 static void drawOverlay(Context *ctx, unsigned int width, unsigned int height) {
+	// if no socket is active, initialize and connect to socket
 	if (ctx->iSocket == -1) {
 		releaseMem(ctx);
 		if (! ctx->saName.sun_path[0])
@@ -259,11 +264,23 @@ static void drawOverlay(Context *ctx, unsigned int width, unsigned int height) {
 			ods("connect() failure %s", ctx->saName.sun_path);
 			return;
 		}
-		ods("Connected");
+		ods("Socket connected");
+
+		struct OverlayMsg om;
+		om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
+		om.omh.uiType = OVERLAY_MSGTYPE_PID;
+		om.omh.iLength = sizeof(struct OverlayMsgPid);
+		om.omp.pid = (unsigned int)getpid(); // getpid can't fail
+
+		if (!sendMessage(ctx, &om))
+			return;
+
+		ods("SentPid");
 	}
 
+	// if overlay size (width or height) is not up-to-date create and send an overlay initialization message
 	if ((ctx->uiWidth != width) || (ctx->uiHeight != height)) {
-		ods("Sent init %i %i", width, height);
+		ods("Sending init overlay msg with w h %i %i", width, height);
 		releaseMem(ctx);
 
 		ctx->uiWidth = width;
@@ -280,8 +297,10 @@ static void drawOverlay(Context *ctx, unsigned int width, unsigned int height) {
 			return;
 	}
 
+	// receive and process overlay messages
 	while (1) {
-		if (ctx->omMsg.omh.iLength == -1) {
+		if (ctx->omMsg.omh.iLength < 0) {
+			// receive the overlay message header
 			ssize_t length = recv(ctx->iSocket, ctx->omMsg.headerbuffer, sizeof(struct OverlayMsgHeader), 0);
 			if (length < 0) {
 				if ((errno == EAGAIN) || (errno == EWOULDBLOCK))
@@ -289,25 +308,28 @@ static void drawOverlay(Context *ctx, unsigned int width, unsigned int height) {
 				disconnect(ctx);
 				return;
 			} else if (length != sizeof(struct OverlayMsgHeader)) {
-				ods("Short header read");
+				ods("Short header read on overlay message");
 				disconnect(ctx);
 				return;
 			}
 		} else {
-			ssize_t  length = recv(ctx->iSocket, ctx->omMsg.msgbuffer, ctx->omMsg.omh.iLength, 0);
+			// receive the overlay message body
+			ssize_t length = recv(ctx->iSocket, ctx->omMsg.msgbuffer, (size_t)ctx->omMsg.omh.iLength, 0);
 			if (length < 0) {
 				if ((errno == EAGAIN) || (errno == EWOULDBLOCK))
 					break;
 				disconnect(ctx);
 				return;
 			} else if (length != ctx->omMsg.omh.iLength) {
-				ods("Short message read %x %ld/%d", ctx->omMsg.omh.uiType, length, ctx->omMsg.omh.iLength);
+				ods("Short overlay message read %x %zd/%d", ctx->omMsg.omh.uiType, length, ctx->omMsg.omh.iLength);
 				disconnect(ctx);
 				return;
 			}
+			// set len to -1 again for a clean state on next receive
 			ctx->omMsg.omh.iLength = -1;
 
 			switch (ctx->omMsg.omh.uiType) {
+				// shared memory overlay message:
 				case OVERLAY_MSGTYPE_SHMEM: {
 						struct OverlayMsgShmem *oms = (struct OverlayMsgShmem *) & ctx->omMsg.omi;
 						ods("SHMEM %s", oms->a_cName);
@@ -315,30 +337,38 @@ static void drawOverlay(Context *ctx, unsigned int width, unsigned int height) {
 						int fd = shm_open(oms->a_cName, O_RDONLY, 0600);
 						if (fd != -1) {
 							struct stat buf;
-							fstat(fd, &buf);
-							if (buf.st_size >= ctx->uiWidth * ctx->uiHeight * 4) {
-								ctx->uiMappedLength = buf.st_size;
-								ctx->a_ucTexture = mmap(NULL, buf.st_size, PROT_READ, MAP_SHARED, fd, 0);
-								if (ctx->a_ucTexture != (unsigned char *) -1) {
-									struct OverlayMsg om;
-									om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
-									om.omh.uiType = OVERLAY_MSGTYPE_SHMEM;
-									om.omh.iLength = 0;
-
-									if (! sendMessage(ctx, &om))
-										return;
-
-									regenTexture(ctx);
-									continue;
+							
+							if (fstat(fd, &buf) != -1) {
+								unsigned int buflen = buf.st_size;
+								if (buflen >= ctx->uiWidth * ctx->uiHeight * 4
+								        && buflen < 512 * 1024 * 1024) {
+									ctx->uiMappedLength = buflen;
+									ctx->a_ucTexture = mmap(NULL, (size_t)buflen, PROT_READ, MAP_SHARED, fd, 0);
+									if (ctx->a_ucTexture != MAP_FAILED) {
+										// mmap successfull; send a new bodyless sharedmemory overlay message and regenerate the overlay texture
+										struct OverlayMsg om;
+										om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
+										om.omh.uiType = OVERLAY_MSGTYPE_SHMEM;
+										om.omh.iLength = 0;
+	
+										if (! sendMessage(ctx, &om))
+											return;
+	
+										regenTexture(ctx);
+										continue;
+									}
+									ctx->a_ucTexture = NULL;
 								}
-								ctx->a_ucTexture = NULL;
+								ctx->uiMappedLength = 0;
+							} else {
+								ods("Failed to fstat memory map");
 							}
-							ctx->uiMappedLength = 0;
 							close(fd);
 						}
 						ods("Failed to map memory");
 					}
 					break;
+				// blit overlay message: blit overlay texture from shared memory to gl-texture var
 				case OVERLAY_MSGTYPE_BLIT: {
 						struct OverlayMsgBlit *omb = & ctx->omMsg.omb;
 						ods("BLIT %d %d %d %d", omb->x, omb->y, omb->w, omb->h);
@@ -347,23 +377,26 @@ static void drawOverlay(Context *ctx, unsigned int width, unsigned int height) {
 
 							if ((omb->x == 0) && (omb->y == 0) && (omb->w == ctx->uiWidth) && (omb->h == ctx->uiHeight)) {
 								ods("Optimzied fullscreen blit");
-								glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, ctx->uiWidth, ctx->uiHeight, 0, GL_BGRA, GL_UNSIGNED_BYTE, ctx->a_ucTexture);
+								glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, (GLsizei)ctx->uiWidth, (GLsizei)ctx->uiHeight, 0, GL_BGRA, GL_UNSIGNED_BYTE, ctx->a_ucTexture);
 							} else {
+								// allocate temporary memory
 								unsigned int x = omb->x;
 								unsigned int y = omb->y;
 								unsigned int w = omb->w;
 								unsigned int h = omb->h;
 								unsigned char *ptr = (unsigned char *) malloc(w*h*4);
-								unsigned int r;
+								unsigned int row;
 								memset(ptr, 0, w * h * 4);
 
-								for (r = 0; r < h; ++r) {
-									const unsigned char *sptr = ctx->a_ucTexture + 4 * ((y+r) * ctx->uiWidth + x);
-									unsigned char *dptr = ptr + 4 * w * r;
+								// copy overlay texture to temporary memory to adapt to full opengl ui size (overlay at correct place)
+								for (row = 0; row < h; ++row) {
+									const unsigned char *sptr = ctx->a_ucTexture + 4 * ((y+row) * ctx->uiWidth + x);
+									unsigned char *dptr = ptr + 4 * w * row;
 									memcpy(dptr, sptr, w * 4);
 								}
 
-								glTexSubImage2D(GL_TEXTURE_2D, 0, x, y, w, h, GL_BGRA, GL_UNSIGNED_BYTE, ptr);
+								// copy temporary texture to opengl
+								glTexSubImage2D(GL_TEXTURE_2D, 0, (GLint)x, (GLint)y, (GLint)w, (GLint)h, GL_BGRA, GL_UNSIGNED_BYTE, ptr);
 								free(ptr);
 							}
 						}
@@ -378,6 +411,20 @@ static void drawOverlay(Context *ctx, unsigned int width, unsigned int height) {
 						ctx->uiBottom = oma->y + oma->h;
 					}
 					break;
+				case OVERLAY_MSGTYPE_INTERACTIVE: {
+#if defined(TARGET_MAC)
+						struct OverlayMsgInteractive *omin = & ctx->omMsg.omin;
+						ods("Interactive %d", omin->state);
+						if (bCursorAvail) {
+							if (omin->state) {
+								oCGDisplayHideCursor(kCGNullDirectDisplay);
+							} else {
+								oCGDisplayShowCursor(kCGNullDirectDisplay);
+							}
+						}
+#endif
+					}
+					break;
 				default:
 					break;
 			}
@@ -387,16 +434,20 @@ static void drawOverlay(Context *ctx, unsigned int width, unsigned int height) {
 	if ((ctx->a_ucTexture == NULL) || (ctx->texture == ~0U))
 		return;
 
+	// texture checks, that our gltexture is still valid and sane
 	if (! glIsTexture(ctx->texture)) {
-		ctx->texture = ~0;
+		ctx->texture = ~0U;
 		ods("Lost texture");
 		regenTexture(ctx);
 	} else {
 		glBindTexture(GL_TEXTURE_2D, ctx->texture);
 		GLfloat bordercolor[4];
 		glGetTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, bordercolor);
-		if (bordercolor[0] != fBorder[0] || bordercolor[1] != fBorder[1] || bordercolor[2] != fBorder[2] || bordercolor[3] != fBorder[3]) {
-			ods("Texture hijacked");
+		if (bordercolor[0] != fBorder[0]
+		    || bordercolor[1] != fBorder[1]
+		    || bordercolor[2] != fBorder[2]
+		    || bordercolor[3] != fBorder[3]) {
+			ods("Texture was hijacked! Texture will be regenerated.");
 			regenTexture(ctx);
 		}
 	}
@@ -412,25 +463,40 @@ static void drawOverlay(Context *ctx, unsigned int width, unsigned int height) {
 	float right  = (float)(ctx->uiRight);
 	float bottom = (float)(ctx->uiBottom);
 
-	float xm = (left) / w;
-	float ym = (top) / h;
-	float xmx = (right) / w;
-	float ymx = (bottom) / h;
+	float xm  = left   / w;
+	float ym  = top    / h;
+	float xmx = right  / w;
+	float ymx = bottom / h;
 
+	GLfloat vertex[] = {
+		left, bottom,
+		left, top,
+		right, top,
 
-	GLfloat vertex[] = {left, bottom,
-			    left, top,
-			    right, top,
-			    right, bottom};
-	GLfloat tex[] = {xm, ymx, xm, ym, xmx, ym, xmx, ymx};
+		left, bottom,
+		right, top,
+		right, bottom
+	};
 	glVertexPointer(2, GL_FLOAT, 0, vertex);
+
+	GLfloat tex[] = {
+		xm, ymx,
+		xm, ym,
+		xmx, ym,
+
+		xm, ymx,
+		xmx, ym,
+		xmx, ymx
+	};
 	glTexCoordPointer(2, GL_FLOAT, 0, tex);
-	glDrawArrays(GL_QUADS, 0, 4);
+
+	glDrawArrays(GL_TRIANGLES, 0, 6);
 
 	glPopMatrix();
 }
 
 static void drawContext(Context * ctx, int width, int height) {
+	// calculate FPS and send it as an overlay message
 	clock_t t = clock();
 	float elapsed = (float)(t - ctx->timeT) / CLOCKS_PER_SEC;
 	++(ctx->frameCount);
@@ -439,7 +505,7 @@ static void drawContext(Context * ctx, int width, int height) {
 		om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
 		om.omh.uiType = OVERLAY_MSGTYPE_FPS;
 		om.omh.iLength = sizeof(struct OverlayMsgFps);
-		om.omf.fps = ctx->frameCount / elapsed;
+		om.omf.fps = (float)ctx->frameCount / elapsed;
 
 		sendMessage(ctx, &om);
 
@@ -447,14 +513,14 @@ static void drawContext(Context * ctx, int width, int height) {
 		ctx->timeT = t;
 	}
 
-	GLint program;
+	GLuint program;
 	GLint viewport[4];
 	int i;
 
 	glPushAttrib(GL_ALL_ATTRIB_BITS);
 	glPushClientAttrib(GL_ALL_ATTRIB_BITS);
 	glGetIntegerv(GL_VIEWPORT, viewport);
-	glGetIntegerv(GL_CURRENT_PROGRAM, &program);
+	glGetIntegerv(GL_CURRENT_PROGRAM, (GLint *)&program);
 
 	glViewport(0, 0, width, height);
 
@@ -471,6 +537,8 @@ static void drawContext(Context * ctx, int width, int height) {
 	glPushMatrix();
 	glLoadIdentity();
 
+	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
+
 	glDisable(GL_ALPHA_TEST);
 	glDisable(GL_AUTO_NORMAL);
 	// Skip clip planes, there are thousands of them.
@@ -529,7 +597,7 @@ static void drawContext(Context * ctx, int width, int height) {
 	glGetIntegerv(GL_MAX_TEXTURE_UNITS, &texunits);
 
 	for (i=texunits-1;i>=0;--i) {
-		glActiveTexture(GL_TEXTURE0 + i);
+		glActiveTexture(GL_TEXTURE0 + (GLenum)i);
 		glDisable(GL_TEXTURE_1D);
 		glDisable(GL_TEXTURE_2D);
 		glDisable(GL_TEXTURE_3D);
@@ -539,6 +607,16 @@ static void drawContext(Context * ctx, int width, int height) {
 	glDisable(GL_VERTEX_PROGRAM_ARB);
 	glDisable(GL_FRAGMENT_PROGRAM_ARB);
 
+	GLint enabled;
+	for (i=0;i<ctx->maxVertexAttribs;++i) {
+		enabled = GL_FALSE;
+		glGetVertexAttribiv((GLuint)i, GL_VERTEX_ATTRIB_ARRAY_ENABLED, &enabled);
+		if (enabled == GL_TRUE) {
+			glDisableVertexAttribArray((GLuint)i);
+			ctx->vertexAttribStates[i] = GL_TRUE;
+		}
+	}
+
 	glUseProgram(ctx->uiProgram);
 
 	glEnable(GL_COLOR_MATERIAL);
@@ -556,21 +634,32 @@ static void drawContext(Context * ctx, int width, int height) {
 	glEnableClientState(GL_VERTEX_ARRAY);
 	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 
-	GLint bound = 0, vbobound = 0;
-	glGetIntegerv(GL_PIXEL_UNPACK_BUFFER_BINDING, &bound);
-	glGetIntegerv(GL_ARRAY_BUFFER_BINDING, &vbobound);
+	GLuint bound = 0, vbobound = 0;
+	glGetIntegerv(GL_PIXEL_UNPACK_BUFFER_BINDING, (GLint *)&bound);
+	glGetIntegerv(GL_ARRAY_BUFFER_BINDING, (GLint *)&vbobound);
 
-	if (bound != 0)
+	if (bound != 0) {
 		glBindBuffer(GL_PIXEL_UNPACK_BUFFER_ARB, 0);
-	if (vbobound != 0)
+	}
+	if (vbobound != 0) {
 		glBindBuffer(GL_ARRAY_BUFFER, 0);
+	}
 
-	drawOverlay(ctx, width, height);
-	
-	if (bound != 0)
+	drawOverlay(ctx, (unsigned int)width, (unsigned int)height);
+
+	if (bound != 0) {
 		glBindBuffer(GL_PIXEL_UNPACK_BUFFER_ARB, bound);
-	if (vbobound != 0)
+	}
+	if (vbobound != 0) {
 		glBindBuffer(GL_ARRAY_BUFFER, vbobound);
+	}
+
+	for (i=0;i<ctx->maxVertexAttribs;++i) {
+		if (ctx->vertexAttribStates[i] == GL_TRUE) {
+			glEnableVertexAttribArray((GLuint)i);
+			ctx->vertexAttribStates[i] = GL_FALSE;
+		}
+	}
 
 	glMatrixMode(GL_TEXTURE);
 	glPopMatrix();
@@ -585,160 +674,13 @@ static void drawContext(Context * ctx, int width, int height) {
 	glPopAttrib();
 	glViewport(viewport[0], viewport[1], viewport[2], viewport[3]);
 	glUseProgram(program);
-	
+
+	// drain opengl error queue
 	while (glGetError() != GL_NO_ERROR);
 }
 
-__attribute__((visibility("default")))
-void glXSwapBuffers(Display * dpy, GLXDrawable draw) {
-	if (!oglXSwapBuffers)
-		resolveOpenGL();
-
-		GLXContext ctx = glXGetCurrentContext();
-
-		Context *c = contexts;
-		while (c) {
-			if ((c->dpy == dpy) && (c->draw == draw))
-				break;
-			c = c->next;
-		}
-
-		if (!c) {
-			ods("Current context is: %p", ctx);
-
-			c = (Context *) malloc(sizeof(Context));
-			if (!c) {
-				ods("malloc failure");
-				return;
-			}
-			memset(c, 0, sizeof(Context));
-			c->next = contexts;
-			c->dpy = dpy;
-			c->draw = draw;
-
-			c->bGlx = false;
-			c->bValid = false;
-
-			int major, minor;
-			if (glXQueryVersion(dpy, &major, &minor)) {
-				ods("GLX version %d.%d", major, minor);
-				c->bValid = true;
-				if ((major > 1) || (major==1 && minor >= 3))
-					c->bGlx = true;
-			}
-			contexts = c;
-			newContext(c);
-		}
-
-		if (c->bValid) {
-			GLuint width, height;
-			if (c->bGlx) {
-				glXQueryDrawable(dpy, draw, GLX_WIDTH, (unsigned int *) &width);
-				glXQueryDrawable(dpy, draw, GLX_HEIGHT, (unsigned int *) &height);
-			} else {
-				GLint viewport[4];
-				glGetIntegerv(GL_VIEWPORT, viewport);
-				width = viewport[2];
-				height = viewport[3];
-			}
-
-			drawContext(c, width, height);
-	}
-	oglXSwapBuffers(dpy, draw);
-}
-
-#define FGRAB(x) if (strcmp((const char *)(func), #x)==0) return (__GLXextFuncPtr)(x);
-
-__attribute__((visibility("default")))
-void (*glXGetProcAddress(const GLubyte * func))(void) {
-	FGRAB(glXSwapBuffers);
-	FGRAB(glXGetProcAddressARB);
-	FGRAB(glXGetProcAddress);
-
-	if (!oglXGetProcAddressARB && !oglXGetProcAddress)
-		resolveOpenGL();
-	if (oglXGetProcAddress)
-		return oglXGetProcAddress(func);
-	else if (oglXGetProcAddressARB)
-		return oglXGetProcAddressARB(func);
-	else
-		return (__GLXextFuncPtr)(odlsym(RTLD_NEXT, (const char *)(func)));
-}
-
-__attribute__((visibility("default")))
-__GLXextFuncPtr glXGetProcAddressARB(const GLubyte * func) {
-	return (void (*)(void)) glXGetProcAddress(func);
-}
-
-__attribute__((constructor))
-static void initializeLibrary() {
-	if (odlsym)
-		return;
-
-	if (getenv("MUMBLE_OVERLAY_DEBUG"))
-		bDebug = true;
-	else
-		bDebug = false;
-
-	ods("Mumble overlay library loaded");
-	void *dl = dlopen("libdl.so.2", RTLD_LAZY);
-	if (!dl) {
-		ods("Failed to open libdl.so.2");
-	} else {
-		int i;
-		struct link_map *lm = (struct link_map *) dl;
-		int nchains = 0;
-		ElfW(Sym) *symtab = NULL;
-		const char *strtab = NULL;
-
-		ElfW(Dyn) *dyn = lm->l_ld;
-
-		while (dyn->d_tag) {
-			switch (dyn->d_tag) {
-				case DT_HASH:
-					nchains = *(int *)(dyn->d_un.d_ptr + 4);
-					break;
-				case DT_STRTAB:
-					strtab = (const char *) dyn->d_un.d_ptr;
-					break;
-				case DT_SYMTAB:
-					symtab = (ElfW(Sym) *) dyn->d_un.d_ptr;
-					break;
-			}
-			dyn ++;
-		}
-		ods("Iterating dlsym table %p %p %d", symtab, strtab, nchains);
-		for (i=0;i<nchains;++i) {
-			// ELF32_ST_TYPE and ELF64_ST_TYPE are the same
-			if (ELF32_ST_TYPE(symtab[i].st_info) != STT_FUNC)
-				continue;
-			if (strcmp(strtab+symtab[i].st_name, "dlsym") == 0)
-				odlsym = (void*)lm->l_addr + symtab[i].st_value;
-		}
-		ods("Original dlsym at %p", odlsym);
-	}
-}
-
-#define OGRAB(name) if (handle == RTLD_DEFAULT) handle = RTLD_NEXT; symbol = odlsym(handle, #name); if (symbol) { o##name = (__typeof__(&name)) symbol; symbol = (void *) name;}
-__attribute__((visibility("default")))
-void *dlsym(void *handle, const char *name) {
-	if (!odlsym)
-		initializeLibrary();
-
-	void *symbol;
-
-	ods("Request for symbol %s (%p:%p)", name, handle, odlsym);
-
-	if (strcmp(name, "glXSwapBuffers") == 0) {
-		OGRAB(glXSwapBuffers);
-	} else if (strcmp(name, "glXGetProcAddress") == 0) {
-		OGRAB(glXGetProcAddress);
-	} else if (strcmp(name, "glXGetProcAddressARB") == 0) {
-		OGRAB(glXGetProcAddressARB);
-	} else if (strcmp(name, "dlsym") == 0) {
-		return (void *) dlsym;
-	} else {
-		symbol = odlsym(handle, name);
-	}
-	return symbol;
-}
+#if defined(TARGET_UNIX)
+# include "init_unix.c"
+#elif defined(TARGET_MAC)
+# include "init_mac.c"
+#endif
