@@ -1,39 +1,11 @@
-/* Copyright (C) 2005-2010, Snares <snares@users.sourceforge.net>
-   Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "../mumble_plugin_win32.h"
 
-BYTE *posptr;
-BYTE *frontptr;
-BYTE *topptr;
+procptr_t posptr, frontptr, topptr;
 
 static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, std::string &, std::wstring &) {
 	for (int i=0;i<3;i++)
@@ -55,12 +27,12 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	/*
 		value is 0 when one is not in a game, 4 when one is
 
-	   ok = peekProc((BYTE *) 0x, &state, 1); // Magical state value
-	   if (! ok)
-	 	return false;
+	ok = peekProc((BYTE *) 0x, &state, 1); // Magical state value
+	if (! ok)
+		return false;
 
-	   if (state == 0)
-	          return true; // This results in all vectors beeing zero which tells Mumble to ignore them.
+	if (state == 0)
+		return true; // This results in all vectors beeing zero which tells Mumble to ignore them.
 	*/
 
 	ok = peekProc(posptr, avatar_pos, 12) &&
@@ -102,7 +74,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 }
 
 static int trylock(const std::multimap<std::wstring, unsigned long long int> &pids) {
-	posptr = NULL;
+	posptr = 0;
 
 	if (! initialize(pids, L"arma2.exe"))
 		return false;
@@ -120,11 +92,11 @@ static int trylock(const std::multimap<std::wstring, unsigned long long int> &pi
 	   if (!pModule)
 	*/
 
-	BYTE *ptr1 = peekProc<BYTE *>((BYTE *) 0x00C500FC);
+	procptr_t ptr1 = peekProcPtr(0x00C500FC);
 
-	BYTE *ptr2 = peekProc<BYTE *>(ptr1 + 0x88);
+	procptr_t ptr2 = peekProcPtr(ptr1 + 0x88);
 
-	BYTE *base = ptr2 + 0x10;
+	procptr_t base = ptr2 + 0x10;
 
 	posptr = base + 0x18;
 	frontptr = base;
@@ -171,10 +143,10 @@ static MumblePlugin2 arma2plug2 = {
 	trylock
 };
 
-extern "C" __declspec(dllexport) MumblePlugin *getMumblePlugin() {
+extern "C" MUMBLE_PLUGIN_EXPORT MumblePlugin *getMumblePlugin() {
 	return &arma2plug;
 }
 
-extern "C" __declspec(dllexport) MumblePlugin2 *getMumblePlugin2() {
+extern "C" MUMBLE_PLUGIN_EXPORT MumblePlugin2 *getMumblePlugin2() {
 	return &arma2plug2;
 }
