@@ -1,33 +1,7 @@
-/* Copyright (C) 2009-2012, Stefan Hacker <dD0t@users.sourceforge.net>
-   Copyright (C) 2005-2012, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "../mumble_plugin_win32.h"
 
@@ -53,7 +27,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 
 			0x0098FD2C		byte	Magical state value
 	*/
-	ok = peekProc((BYTE *) 0x0098FD2C, &state, 1); // Magical state value
+	ok = peekProc(0x0098FD2C, &state, 1); // Magical state value
 	if (! ok)
 		return false;
 	/*
@@ -68,11 +42,11 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	if (state != 4)
 		return true; // This results in all vectors beeing zero which tells mumble to ignore them.
 
-	ok = peekProc((BYTE *) 0x008DE23C, avatar_pos+2, 4) &&	//Z
-	     peekProc((BYTE *) 0x008DE234, avatar_pos, 4) &&	//X
-	     peekProc((BYTE *) 0x008DE238, avatar_pos+1, 4) && //Y
-	     peekProc((BYTE *) 0x008DE244, &viewHor, 4) && //Hor
-	     peekProc((BYTE *) 0x008DE240, &viewVer, 4); //Ver
+	ok = peekProc(0x008DE23C, avatar_pos+2, 4) &&	//Z
+	     peekProc(0x008DE234, avatar_pos, 4) &&	//X
+	     peekProc(0x008DE238, avatar_pos+1, 4) && //Y
+	     peekProc(0x008DE244, &viewHor, 4) && //Hor
+	     peekProc(0x008DE240, &viewVer, 4); //Ver
 
 	if (! ok)
 		return false;
@@ -95,15 +69,15 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 
 	// Calculate view unit vector
 	/*
-	   Vertical view 0 when centered
-					85	when looking down
-				   275 when looking up
+	   Vertical view 0° when centered
+					85°	when looking down
+				   275° when looking up
 	   Decreasing when looking up.
 
-	   Horizontal is 0 when facing North
-					90 when facing West
-				   180 when facing South
-				   270 when facing East
+	   Horizontal is 0° when facing North
+					90° when facing West
+				   180° when facing South
+				   270° when facing East
 	   Increasing when turning left.
 	*/
 	viewVer *= static_cast<float>(M_PI / 180.0f);
@@ -167,10 +141,10 @@ static MumblePlugin2 cod5plug2 = {
 	trylock
 };
 
-extern "C" __declspec(dllexport) MumblePlugin *getMumblePlugin() {
+extern "C" MUMBLE_PLUGIN_EXPORT MumblePlugin *getMumblePlugin() {
 	return &cod5plug;
 }
 
-extern "C" __declspec(dllexport) MumblePlugin2 *getMumblePlugin2() {
+extern "C" MUMBLE_PLUGIN_EXPORT MumblePlugin2 *getMumblePlugin2() {
 	return &cod5plug2;
 }
