@@ -1,3 +1,8 @@
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
+
 /*
  Copyright (c) 2009-2011 Ilmar 'Ingaras' Kruis (seaeagle1@users.sourceforge.net)
 
@@ -40,7 +45,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	byte l[2];
 	byte r,i;
 	float o[3];
-	BYTE *hPtr;
+	procptr_t hPtr;
 	float h;
 
 	/*
@@ -61,16 +66,16 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 		nPtr = pointer to character name (unique on a server)
 	*/
 
-	ok = peekProc((BYTE *) 0x01272D34, o, 12) &&
-	     peekProc((BYTE *) 0x01272D2C, l, 2) &&
-	     peekProc((BYTE *) 0x01272D28, &r, 1) &&
-	     peekProc((BYTE *) 0x01272D20, &i, 1) &&
-	     peekProc((BYTE *)(pModule + 0x00A138A4), &hPtr, 4);
+	ok = peekProc(0x01272D34, o, 12) &&
+	     peekProc(0x01272D2C, l, 2) &&
+	     peekProc(0x01272D28, &r, 1) &&
+	     peekProc(0x01272D20, &i, 1) &&
+	     peekProc(pModule + 0x00A138A4, &hPtr, 4);
 
 	if (! ok)
 		return false;
 
-	ok = peekProc((BYTE *)(hPtr  + 0x0000046F), &h, 4);
+	ok = peekProc((procptr_t)(hPtr  + 0x0000046F), &h, 4);
 
 	if (! ok)
 		return false;
@@ -166,10 +171,10 @@ static MumblePlugin2 lotroplug2 = {
 	trylock
 };
 
-extern "C" __declspec(dllexport) MumblePlugin *getMumblePlugin() {
+extern "C" MUMBLE_PLUGIN_EXPORT MumblePlugin *getMumblePlugin() {
 	return &lotroplug;
 }
 
-extern "C" __declspec(dllexport) MumblePlugin2 *getMumblePlugin2() {
+extern "C" MUMBLE_PLUGIN_EXPORT MumblePlugin2 *getMumblePlugin2() {
 	return &lotroplug2;
 }
