@@ -1,37 +1,13 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
-
-#ifndef CONNECTION_H_
-#define CONNECTION_H_
+#ifndef MUMBLE_CONNECTION_H_
+#define MUMBLE_CONNECTION_H_
 
 #include <QtCore/QtGlobal>
+#include <QtCore/QMutex>
 #if QT_VERSION >= 0x040700
 #include <QtCore/QElapsedTimer>
 #else
@@ -89,15 +65,25 @@ class Connection : public QObject {
 		void sendMessage(const QByteArray &qbaMsg);
 		void disconnectSocket(bool force=false);
 		void forceFlush();
-		int activityTime() const;
+		qint64 activityTime() const;
 		void resetActivityTime();
 
+#ifdef MURMUR
+		/// qmCrypt locks access to csCrypt.
+		QMutex qmCrypt;
+#endif
 		CryptState csCrypt;
 
 		QList<QSslCertificate> peerCertificateChain() const;
 		QSslCipher sessionCipher() const;
+		QSsl::SslProtocol sessionProtocol() const;
+		QString sessionProtocolString() const;
 		QHostAddress peerAddress() const;
 		quint16 peerPort() const;
+		/// Look up the local address of this Connection.
+		QHostAddress localAddress() const;
+		/// Look up the local port of this Connection.
+		quint16 localPort() const;
 		bool bDisconnectedEmitted;
 
 		void setToS();
@@ -106,6 +92,8 @@ class Connection : public QObject {
 #endif
 };
 
+#if QT_VERSION < 0x050000
 Q_DECLARE_METATYPE(QAbstractSocket::SocketError)
+#endif
 
 #endif
