@@ -1,34 +1,13 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
-
-#include "murmur_pch.h"
+#ifdef MUMBLE
+	#include "mumble_pch.hpp"
+#else
+	#include "murmur_pch.h"
+#endif
 
 #include "Group.h"
 
@@ -135,28 +114,28 @@ bool Group::isMember(Channel *curChan, Channel *aclChan, QString name, ServerUse
 	bool hash = false;
 	c = curChan;
 
-	while (1) {
+	while (true) {
 		if (name.isEmpty())
 			return false;
 
-		if (name.startsWith(QChar::fromAscii('!'))) {
+		if (name.startsWith(QChar::fromLatin1('!'))) {
 			invert = true;
 			name = name.remove(0,1);
 			continue;
 		}
 
-		if (name.startsWith(QChar::fromAscii('~'))) {
+		if (name.startsWith(QChar::fromLatin1('~'))) {
 			c = aclChan;
 			name = name.remove(0,1);
 			continue;
 		}
 
-		if (name.startsWith(QChar::fromAscii('#'))) {
+		if (name.startsWith(QChar::fromLatin1('#'))) {
 			token = true;
 			name = name.remove(0,1);
 			continue;
 		}
-		if (name.startsWith(QChar::fromAscii('$'))) {
+		if (name.startsWith(QChar::fromLatin1('$'))) {
 			hash = true;
 			name = name.remove(0,1);
 			continue;
@@ -181,22 +160,22 @@ bool Group::isMember(Channel *curChan, Channel *aclChan, QString name, ServerUse
 		m = (pl->cChannel == c);
 	else if (name == QLatin1String("out"))
 		m = !(pl->cChannel == c);
-	else if (name.startsWith(QLatin1String("sub"))) {
+	else if (name == QLatin1String("sub")
+			|| name.startsWith(QLatin1String("sub,"))) {
+		
 		name = name.remove(0,4);
 		int mindesc = 1;
 		int maxdesc = 1000;
 		int minpath = 0;
 		QStringList args = name.split(QLatin1String(","));
-		switch (args.count()) {
-			default:
-			case 3:
-				maxdesc = args[2].isEmpty() ? maxdesc : args[2].toInt();
-			case 2:
-				mindesc = args[1].isEmpty() ? mindesc : args[1].toInt();
-			case 1:
-				minpath = args[0].isEmpty() ? minpath : args[0].toInt();
-			case 0:
-				break;
+		if (args.count() >= 3) {
+			maxdesc = args[2].isEmpty() ? maxdesc : args[2].toInt();
+		}
+		if (args.count() >= 2) {
+			mindesc = args[1].isEmpty() ? mindesc : args[1].toInt();
+		}
+		if (args.count() >= 1) {
+			minpath = args[0].isEmpty() ? minpath : args[0].toInt();
 		}
 
 		Channel *home = pl->cChannel;
