@@ -1,32 +1,7 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "mumble_pch.hpp"
 
@@ -34,6 +9,7 @@
 
 #include "AudioOutput.h"
 #include "CELTCodec.h"
+#include "OpusCodec.h"
 #include "ServerHandler.h"
 #include "MainWindow.h"
 #include "User.h"
@@ -44,8 +20,10 @@
 #include "NetworkConfig.h"
 #include "VoiceRecorder.h"
 
-#ifdef USE_OPUS
-#include "opus.h"
+#ifdef USE_RNNOISE
+extern "C" {
+#include "rnnoise.h"
+}
 #endif
 
 // Remember that we cannot use static member classes that are not pointers, as the constructor
@@ -103,6 +81,9 @@ AudioInput::AudioInput() : opusBuffer(g.s.iFramesPerPacket * (SAMPLE_RATE / 100)
 
 	umtType = MessageHandler::UDPVoiceCELTAlpha;
 
+	activityState = ActivityStateActive;
+	oCodec = NULL;
+	opusState = NULL;
 	cCodec = NULL;
 	ceEncoder = NULL;
 
@@ -110,8 +91,22 @@ AudioInput::AudioInput() : opusBuffer(g.s.iFramesPerPacket * (SAMPLE_RATE / 100)
 	iFrameSize = SAMPLE_RATE / 100;
 
 #ifdef USE_OPUS
-	opusState = opus_encoder_create(SAMPLE_RATE, 1, OPUS_APPLICATION_VOIP, NULL);
-	opus_encoder_ctl(opusState, OPUS_SET_VBR(0)); // CBR
+	oCodec = g.oCodec;
+	if (oCodec) {
+		if (!g.s.bUseOpusMusicEncoding) {
+			opusState = oCodec->opus_encoder_create(SAMPLE_RATE, 1, OPUS_APPLICATION_VOIP, NULL);
+			qWarning("AudioInput: Opus encoder set for VOIP");
+		} else {
+			opusState = oCodec->opus_encoder_create(SAMPLE_RATE, 1, OPUS_APPLICATION_AUDIO, NULL);
+			qWarning("AudioInput: Opus encoder set for Music");
+		}
+
+		oCodec->opus_encoder_ctl(opusState, OPUS_SET_VBR(0)); // CBR
+	}
+#endif
+
+#ifdef USE_RNNOISE
+	denoiseState = rnnoise_create();
 #endif
 
 	qWarning("AudioInput: %d bits/s, %d hz, %d sample", iAudioQuality, iSampleRate, iFrameSize);
@@ -166,8 +161,15 @@ AudioInput::~AudioInput() {
 	wait();
 
 #ifdef USE_OPUS
-	if (opusState)
-		opus_encoder_destroy(opusState);
+	if (opusState) {
+		oCodec->opus_encoder_destroy(opusState);
+	}
+#endif
+
+#ifdef USE_RNNOISE
+	if (denoiseState) {
+		rnnoise_destroy(denoiseState);
+	}
 #endif
 
 	if (ceEncoder) {
@@ -201,12 +203,13 @@ bool AudioInput::isTransmitting() const {
 };
 
 #define IN_MIXER_FLOAT(channels) \
-static void inMixerFloat##channels ( float * RESTRICT buffer, const void * RESTRICT ipt, unsigned int nsamp, unsigned int N) { \
+static void inMixerFloat##channels ( float * RESTRICT buffer, const void * RESTRICT ipt, unsigned int nsamp, unsigned int N, quint64 mask) { \
   const float * RESTRICT input = reinterpret_cast<const float *>(ipt); \
-  register const float m = 1.0f / static_cast<float>(channels); \
+  const float m = 1.0f / static_cast<float>(channels); \
   Q_UNUSED(N); \
+  Q_UNUSED(mask); \
   for(unsigned int i=0;i<nsamp;++i) {\
-	  register float v= 0.0f; \
+	  float v= 0.0f; \
 	  for(unsigned int j=0;j<channels;++j) \
 	  	v += input[i*channels+j]; \
 	  buffer[i] = v * m; \
@@ -214,18 +217,65 @@ static void inMixerFloat##channels ( float * RESTRICT buffer, const void * RESTR
 }
 
 #define IN_MIXER_SHORT(channels) \
-static void inMixerShort##channels ( float * RESTRICT buffer, const void * RESTRICT ipt, unsigned int nsamp, unsigned int N) { \
+static void inMixerShort##channels ( float * RESTRICT buffer, const void * RESTRICT ipt, unsigned int nsamp, unsigned int N, quint64 mask) { \
   const short * RESTRICT input = reinterpret_cast<const short *>(ipt); \
-  register const float m = 1.0f / (32768.f * static_cast<float>(channels)); \
+  const float m = 1.0f / (32768.f * static_cast<float>(channels)); \
   Q_UNUSED(N); \
+  Q_UNUSED(mask); \
   for(unsigned int i=0;i<nsamp;++i) {\
-	  register float v= 0.0f; \
+	  float v= 0.0f; \
 	  for(unsigned int j=0;j<channels;++j) \
 	  	v += static_cast<float>(input[i*channels+j]); \
 	  buffer[i] = v * m; \
   } \
 }
 
+static void inMixerFloatMask(float * RESTRICT buffer, const void * RESTRICT ipt, unsigned int nsamp, unsigned int N, quint64 mask) { \
+	const float * RESTRICT input = reinterpret_cast<const float *>(ipt);
+
+	unsigned int chancount = 0;
+	STACKVAR(unsigned int, chanindex, N);
+	for (unsigned int j = 0; j < N; ++j) {
+		if ((mask & (1ULL << j)) == 0) {
+			continue;
+		}
+		chanindex[chancount] = j; // Use chancount as index into chanindex.
+		++chancount;
+	}
+
+	const float m = 1.0f / static_cast<float>(chancount);
+	for(unsigned int i = 0; i < nsamp; ++i) {
+		float v = 0.0f;
+		for(unsigned int j = 0; j < chancount; ++j) {
+			v += input[i * N + chanindex[j]];
+		}
+		buffer[i] = v * m;
+	}
+}
+
+static void inMixerShortMask(float * RESTRICT buffer, const void * RESTRICT ipt, unsigned int nsamp, unsigned int N, quint64 mask) {
+	const short * RESTRICT input = reinterpret_cast<const short *>(ipt);
+
+	unsigned int chancount = 0;
+	STACKVAR(unsigned int, chanindex, N);
+	for (unsigned int j = 0; j < N; ++j) {
+		if ((mask & (1ULL << j)) == 0) {
+			continue;
+		}
+		chanindex[chancount] = j; // Use chancount as index into chanindex.
+		++chancount;
+	}
+
+	const float m = 1.0f / static_cast<float>(chancount);
+	for(unsigned int i = 0; i < nsamp; ++i) {
+		float v = 0.0f;
+		for(unsigned int j = 0; j < chancount; ++j) {
+			v += static_cast<float>(input[i * N + chanindex[j]]);
+		}
+		buffer[i] = v * m;
+	}
+}
+
 IN_MIXER_FLOAT(1)
 IN_MIXER_FLOAT(2)
 IN_MIXER_FLOAT(3)
@@ -246,8 +296,18 @@ IN_MIXER_SHORT(7)
 IN_MIXER_SHORT(8)
 IN_MIXER_SHORT(N)
 
-AudioInput::inMixerFunc AudioInput::chooseMixer(const unsigned int nchan, SampleFormat sf) {
+AudioInput::inMixerFunc AudioInput::chooseMixer(const unsigned int nchan, SampleFormat sf, quint64 chanmask) {
 	inMixerFunc r = NULL;
+
+	if (chanmask != 0xffffffffffffffffULL) {
+		if (sf == SampleFloat) {
+			r = inMixerFloatMask;
+		} else if (sf == SampleShort) {
+			r = inMixerShortMask;
+		}
+		return r;
+	}
+
 	if (sf == SampleFloat) {
 		switch (nchan) {
 			case 1:
@@ -344,8 +404,13 @@ void AudioInput::initializeMixer() {
 		pfEchoInput = NULL;
 	}
 
-	imfMic = chooseMixer(iMicChannels, eMicFormat);
-	imfEcho = chooseMixer(iEchoChannels, eEchoFormat);
+	uiMicChannelMask = g.s.uiAudioInputChannelMask;
+
+	// There is no channel mask setting for the echo canceller, so allow all channels.
+	uiEchoChannelMask = 0xffffffffffffffffULL;
+
+	imfMic = chooseMixer(iMicChannels, eMicFormat, uiMicChannelMask);
+	imfEcho = chooseMixer(iEchoChannels, eEchoFormat, uiEchoChannelMask);
 
 	iMicSampleSize = static_cast<int>(iMicChannels * ((eMicFormat == SampleFloat) ? sizeof(float) : sizeof(short)));
 	iEchoSampleSize = static_cast<int>(iEchoChannels * ((eEchoFormat == SampleFloat) ? sizeof(float) : sizeof(short)));
@@ -353,6 +418,9 @@ void AudioInput::initializeMixer() {
 	bResetProcessor = true;
 
 	qWarning("AudioInput: Initialized mixer for %d channel %d hz mic and %d channel %d hz echo", iMicChannels, iMicFreq, iEchoChannels, iEchoFreq);
+	if (uiMicChannelMask != 0xffffffffffffffffULL) {
+		qWarning("AudioInput: using mic channel mask 0x%llx", static_cast<unsigned long long>(uiMicChannelMask));
+	}
 }
 
 void AudioInput::addMic(const void *data, unsigned int nsamp) {
@@ -361,7 +429,7 @@ void AudioInput::addMic(const void *data, unsigned int nsamp) {
 		const unsigned int left = qMin(nsamp, iMicLength - iMicFilled);
 
 		// Append mix into pfMicInput frame buffer (converts 16bit pcm->float if necessary)
-		imfMic(pfMicInput + iMicFilled, data, left, iMicChannels);
+		imfMic(pfMicInput + iMicFilled, data, left, iMicChannels, uiMicChannelMask);
 
 		iMicFilled += left;
 		nsamp -= left;
@@ -447,7 +515,7 @@ void AudioInput::addEcho(const void *data, unsigned int nsamp) {
 			}
 		} else {
 			// Mix echo channels (converts 16bit PCM -> float if needed)
-			imfEcho(pfEchoInput + iEchoFilled, data, left, iEchoChannels);
+			imfEcho(pfEchoInput + iEchoFilled, data, left, iEchoChannels, uiEchoChannelMask);
 		}
 
 		iEchoFilled += left;
@@ -676,27 +744,31 @@ bool AudioInput::selectCodec() {
 	return true;
 }
 
-int AudioInput::encodeOpusFrame(short *source, int size, unsigned char *buffer) {
-	int len = 0;
+int AudioInput::encodeOpusFrame(short *source, int size, EncodingOutputBuffer& buffer) {
+	int len;
 #ifdef USE_OPUS
+	if (!oCodec) {
+		return 0;
+	}
+
 	if (bResetEncoder) {
-		opus_encoder_ctl(opusState, OPUS_RESET_STATE, NULL);
+		oCodec->opus_encoder_ctl(opusState, OPUS_RESET_STATE, NULL);
 		bResetEncoder = false;
 	}
 
-	opus_encoder_ctl(opusState, OPUS_SET_BITRATE(iAudioQuality));
+	oCodec->opus_encoder_ctl(opusState, OPUS_SET_BITRATE(iAudioQuality));
 
-	len = opus_encode(opusState, source, size, buffer, 512);
+	len = oCodec->opus_encode(opusState, source, size, &buffer[0], static_cast<opus_int32>(buffer.size()));
 	const int tenMsFrameCount = (size / iFrameSize);
 	iBitrate = (len * 100 * 8) / tenMsFrameCount;
 #endif
 	return len;
 }
 
-int AudioInput::encodeCELTFrame(short *psSource, unsigned char *buffer) {
-	int len = 0;
+int AudioInput::encodeCELTFrame(short *psSource, EncodingOutputBuffer& buffer) {
+	int len;
 	if (!cCodec)
-		return len;
+		return 0;
 
 	if (bResetEncoder) {
 		cCodec->celt_encoder_ctl(ceEncoder, CELT_RESET_STATE);
@@ -706,7 +778,7 @@ int AudioInput::encodeCELTFrame(short *psSource, unsigned char *buffer) {
 	cCodec->celt_encoder_ctl(ceEncoder, CELT_SET_PREDICTION(0));
 
 	cCodec->celt_encoder_ctl(ceEncoder, CELT_SET_VBR_RATE(iAudioQuality));
-	len = cCodec->encode(ceEncoder, psSource, buffer, qMin(iAudioQuality / (8 * 100), 127));
+	len = cCodec->encode(ceEncoder, psSource, &buffer[0], qMin<int>(iAudioQuality / (8 * 100), static_cast<int>(buffer.size())));
 	iBitrate = len * 100 * 8;
 
 	return len;
@@ -714,7 +786,6 @@ int AudioInput::encodeCELTFrame(short *psSource, unsigned char *buffer) {
 
 void AudioInput::encodeAudioFrame() {
 	int iArg;
-	ClientUser *p=ClientUser::get(g.uiSession);
 	int i;
 	float sum;
 	short max;
@@ -723,17 +794,23 @@ void AudioInput::encodeAudioFrame() {
 
 	iFrameCounter++;
 
+	// As g.iTarget is not protected by any locks, we avoid race-conditions by
+	// copying it once at this point and stick to whatever value it is here. Thus
+	// if the value of g.iTarget changes during the execution of this function,
+	// it won't cause any inconsistencies and the change is reflected once this
+	// function is called again.
+	int voiceTargetID = g.iTarget;
+
 	if (! bRunning)
 		return;
 
 	sum=1.0f;
-	for (i=0;i<iFrameSize;i++)
-		sum += static_cast<float>(psMic[i] * psMic[i]);
-	dPeakMic = qMax(20.0f*log10f(sqrtf(sum / static_cast<float>(iFrameSize)) / 32768.0f), -96.0f);
-
 	max = 1;
-	for (i=0;i<iFrameSize;i++)
-		max = static_cast<short>(abs(psMic[i]) > max ? abs(psMic[i]) : max);
+	for (i=0;i<iFrameSize;i++) {
+		sum += static_cast<float>(psMic[i] * psMic[i]);
+		max = std::max(static_cast<short>(abs(psMic[i])), max);
+	}
+	dPeakMic = qMax(20.0f*log10f(sqrtf(sum / static_cast<float>(iFrameSize)) / 32768.0f), -96.0f);
 	dMaxMic = max;
 
 	if (psSpeaker && (iEchoChannels > 0)) {
@@ -748,6 +825,22 @@ void AudioInput::encodeAudioFrame() {
 	QMutexLocker l(&qmSpeex);
 	resetAudioProcessor();
 
+#ifdef USE_RNNOISE
+	// At the time of writing this code, RNNoise only supports a sample rate of 48000 Hz.
+	if (g.s.bDenoise && denoiseState && (iFrameSize == 480)) {
+		float denoiseFrames[480];
+		for (int i = 0; i < 480; i++) {
+			denoiseFrames[i] = psMic[i];
+		}
+
+		rnnoise_process_frame(denoiseState, denoiseFrames, denoiseFrames);
+
+		for (int i = 0; i < 480; i++) {
+			psMic[i] = denoiseFrames[i];
+		}
+	}
+#endif
+
 	speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_GET_AGC_GAIN, &iArg);
 	float gainValue = static_cast<float>(iArg);
 	iArg = g.s.iNoiseSuppress - iArg;
@@ -778,10 +871,13 @@ void AudioInput::encodeAudioFrame() {
 
 	bool bIsSpeech = false;
 
-	if (level > g.s.fVADmax)
+	if (level > g.s.fVADmax) {
+		// Voice-activation threshold has been reached
 		bIsSpeech = true;
-	else if (level > g.s.fVADmin && bPreviousVoice)
+	} else if (level > g.s.fVADmin && bPreviousVoice) {
+		// Voice-deactivation threshold has not yet been reached
 		bIsSpeech = true;
+	}
 
 	if (! bIsSpeech) {
 		iHoldFrames++;
@@ -791,14 +887,20 @@ void AudioInput::encodeAudioFrame() {
 		iHoldFrames = 0;
 	}
 
-	if (g.s.atTransmit == Settings::Continous)
+	if (g.s.atTransmit == Settings::Continuous) {
+		// Continous transmission is enabled
 		bIsSpeech = true;
-	else if (g.s.atTransmit == Settings::PushToTalk)
+	} else if (g.s.atTransmit == Settings::PushToTalk) {
+		// PTT is enabled, so check if it is currently active
 		bIsSpeech = g.s.uiDoublePush && ((g.uiDoublePush < g.s.uiDoublePush) || (g.tDoublePush.elapsed() < g.s.uiDoublePush));
+	}
 
+	// If g.iPushToTalk > 0 that means that we are currently in some sort of PTT action. For
+	// instance this could mean we're currently whispering
 	bIsSpeech = bIsSpeech || (g.iPushToTalk > 0);
 
-	if (g.s.bMute || ((g.s.lmLoopMode != Settings::Local) && p && (p->bMute || p->bSuppress)) || g.bPushToMute || (g.iTarget < 0)) {
+	ClientUser *p = ClientUser::get(g.uiSession);
+	if (g.s.bMute || ((g.s.lmLoopMode != Settings::Local) && p && (p->bMute || p->bSuppress)) || g.bPushToMute || (voiceTargetID < 0)) {
 		bIsSpeech = false;
 	}
 
@@ -813,7 +915,7 @@ void AudioInput::encodeAudioFrame() {
 	if (p) {
 		if (! bIsSpeech)
 			p->setTalking(Settings::Passive);
-		else if (g.iTarget == 0)
+		else if (voiceTargetID == 0)
 			p->setTalking(Settings::Talking);
 		else
 			p->setTalking(Settings::Shouting);
@@ -830,17 +932,27 @@ void AudioInput::encodeAudioFrame() {
 	if (! bIsSpeech && ! bPreviousVoice) {
 		iBitrate = 0;
 
-		if (g.s.iaeIdleAction != Settings::Nothing && ((tIdle.elapsed() / 1000000ULL) > g.s.iIdleTime)) {
-
+		if ((tIdle.elapsed() / 1000000ULL) > g.s.iIdleTime) {
+			activityState = ActivityStateIdle;
+			tIdle.restart();
 			if (g.s.iaeIdleAction == Settings::Deafen && !g.s.bDeaf) {
-				tIdle.restart();
 				emit doDeaf();
 			} else if (g.s.iaeIdleAction == Settings::Mute && !g.s.bMute) {
-				tIdle.restart();
 				emit doMute();
 			}
 		}
 
+		if (activityState == ActivityStateReturnedFromIdle) {
+			activityState = ActivityStateActive;
+			if (g.s.iaeIdleAction != Settings::Nothing && g.s.bUndoIdleActionUponActivity) {
+				if (g.s.iaeIdleAction == Settings::Deafen && g.s.bDeaf) {
+					emit doDeaf();
+				} else if (g.s.iaeIdleAction == Settings::Mute && g.s.bMute) {
+					emit doMute();
+				}
+			}
+		}
+
 		spx_int32_t increment = 0;
 		speex_preprocess_ctl(sppPreprocess, SPEEX_PREPROCESS_SET_AGC_INCREMENT, &increment);
 		return;
@@ -855,8 +967,10 @@ void AudioInput::encodeAudioFrame() {
 
 	tIdle.restart();
 
-	unsigned char buffer[512];
-	int len;
+	EncodingOutputBuffer buffer;
+	Q_ASSERT(buffer.size() >= static_cast<size_t>(iAudioQuality / 100 * iAudioFrames / 8));
+	
+	int len = 0;
 
 	bool encoded = true;
 	if (!selectCodec())
@@ -864,8 +978,11 @@ void AudioInput::encodeAudioFrame() {
 
 	if (umtType == MessageHandler::UDPVoiceCELTAlpha || umtType == MessageHandler::UDPVoiceCELTBeta) {
 		len = encodeCELTFrame(psSource, buffer);
-		if (len == 0)
+		if (len <= 0) {
+			iBitrate = 0;
+			qWarning() << "encodeCELTFrame failed" << iBufferedFrames << iFrameSize << len;
 			return;
+		}
 		++iBufferedFrames;
 	} else if (umtType == MessageHandler::UDPVoiceOpus) {
 		encoded = false;
@@ -875,24 +992,32 @@ void AudioInput::encodeAudioFrame() {
 		if (!bIsSpeech || iBufferedFrames >= iAudioFrames) {
 			if (iBufferedFrames < iAudioFrames) {
 				// Stuff frame to framesize if speech ends and we don't have enough audio
-				const size_t missingFrames = iAudioFrames - iBufferedFrames;
+				// this way we are guaranteed to have a valid framecount and won't cause
+				// a codec configuration switch by suddenly using a wildly different
+				// framecount per packet.
+				const int missingFrames = iAudioFrames - iBufferedFrames;
 				opusBuffer.insert(opusBuffer.end(), iFrameSize * missingFrames, 0);
 				iBufferedFrames += missingFrames;
+				iFrameCounter += missingFrames;
 			}
+			
+			Q_ASSERT(iBufferedFrames == iAudioFrames);
 
 			len = encodeOpusFrame(&opusBuffer[0], iBufferedFrames * iFrameSize, buffer);
 			opusBuffer.clear();
 			if (len <= 0) {
 				iBitrate = 0;
 				qWarning() << "encodeOpusFrame failed" << iBufferedFrames << iFrameSize << len;
+				iBufferedFrames = 0; // These are lost. Make sure not to mess up our sequence counter next flushCheck.
 				return;
 			}
 			encoded = true;
 		}
 	}
 
-	if (encoded)
-		flushCheck(QByteArray(reinterpret_cast<const char *>(buffer), len), ! bIsSpeech);
+	if (encoded) {
+		flushCheck(QByteArray(reinterpret_cast<char *>(&buffer[0]), len), !bIsSpeech, voiceTargetID);
+	}
 
 	if (! bIsSpeech)
 		iBitrate = 0;
@@ -914,16 +1039,30 @@ static void sendAudioFrame(const char *data, PacketDataStream &pds) {
 		sh->sendMessage(data, pds.size() + 1);
 }
 
-void AudioInput::flushCheck(const QByteArray &frame, bool terminator) {
+void AudioInput::flushCheck(const QByteArray &frame, bool terminator, int voiceTargetID) {
 	qlFrames << frame;
 
 	if (! terminator && iBufferedFrames < iAudioFrames)
 		return;
 
-	int flags = g.iTarget;
-	if (terminator)
+	int flags = 0;
+	if (voiceTargetID > 0) {
+		flags = voiceTargetID;
+	}
+	if (terminator && g.iPrevTarget > 0) {
+		// If we have been whispering to some target but have just ended, terminator will be true. However
+		// in the case of whispering this means that we just released the whisper key so this here is the
+		// last audio frame that is sent for whispering. The whisper key being released means that g.iTarget
+		// is reset to 0 by now. In order to send the last whisper frame correctly, we have to use
+		// g.iPrevTarget which is set to whatever g.iTarget has been before its last change.
+
 		flags = g.iPrevTarget;
 
+		// We reset g.iPrevTarget as it has fulfilled its purpose for this whisper-action. It'll be set
+		// accordingly once the client whispers for the next time.
+		g.iPrevTarget = 0;
+	}
+
 	if (g.s.lmLoopMode == Settings::Server)
 		flags = 0x1f; // Server loopback
 
