@@ -1,37 +1,13 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
-
-#ifndef AUDIOINPUT_H_
-#define AUDIOINPUT_H_
+#ifndef MUMBLE_MUMBLE_AUDIOINPUT_H_
+#define MUMBLE_MUMBLE_AUDIOINPUT_H_
 
 #include <boost/shared_ptr.hpp>
+#include <boost/array.hpp>
 #include <speex/speex.h>
 #include <speex/speex_echo.h>
 #include <speex/speex_preprocess.h>
@@ -47,8 +23,10 @@
 
 class AudioInput;
 class CELTCodec;
+class OpusCodec;
 struct CELTEncoder;
 struct OpusEncoder;
+struct DenoiseState;
 typedef boost::shared_ptr<AudioInput> AudioInputPtr;
 
 class AudioInputRegistrar {
@@ -82,7 +60,7 @@ class AudioInput : public QThread {
 	protected:
 		typedef enum { CodecCELT, CodecSpeex } CodecFormat;
 		typedef enum { SampleShort, SampleFloat } SampleFormat;
-		typedef void (*inMixerFunc)(float * RESTRICT, const void * RESTRICT, unsigned int, unsigned int);
+		typedef void (*inMixerFunc)(float * RESTRICT, const void * RESTRICT, unsigned int, unsigned int, quint64);
 	private:
 		SpeexResamplerState *srsMic, *srsEcho;
 
@@ -93,14 +71,18 @@ class AudioInput : public QThread {
 
 		unsigned int iMicFilled, iEchoFilled;
 		inMixerFunc imfMic, imfEcho;
-		inMixerFunc chooseMixer(const unsigned int nchan, SampleFormat sf);
+		inMixerFunc chooseMixer(const unsigned int nchan, SampleFormat sf, quint64 mask);
 		void resetAudioProcessor();
 
+		OpusCodec *oCodec;
 		OpusEncoder *opusState;
+		DenoiseState *denoiseState;
 		bool selectCodec();
-		int encodeOpusFrame(short *source, int size, unsigned char *buffer);
-		int encodeSpeexFrame(short *pSource, unsigned char *buffer);
-		int encodeCELTFrame(short *pSource, unsigned char *buffer);
+		
+		typedef boost::array<unsigned char, 960> EncodingOutputBuffer;
+		
+		int encodeOpusFrame(short *source, int size, EncodingOutputBuffer& buffer);
+		int encodeCELTFrame(short *pSource, EncodingOutputBuffer& buffer);
 	protected:
 		MessageHandler::UDPMessageType umtType;
 		SampleFormat eMicFormat, eEchoFormat;
@@ -111,6 +93,7 @@ class AudioInput : public QThread {
 		unsigned int iMicLength, iEchoLength;
 		unsigned int iMicSampleSize, iEchoSampleSize;
 		unsigned int iEchoMCLength, iEchoFrameSize;
+		quint64 uiMicChannelMask, uiEchoChannelMask;
 
 		bool bEchoMulti;
 		int	iFrameSize;
@@ -127,7 +110,9 @@ class AudioInput : public QThread {
 		/// needs to be reset.
 		bool bResetEncoder;
 
+		/// Encoded audio rate in bit/s
 		int iAudioQuality;
+		/// Number of 10ms audio "frames" per packet (!= frames in packet)
 		int iAudioFrames;
 
 		short *psMic;
@@ -153,7 +138,7 @@ class AudioInput : public QThread {
 		int iBufferedFrames;
 
 		QList<QByteArray> qlFrames;
-		void flushCheck(const QByteArray &, bool terminator);
+		void flushCheck(const QByteArray &, bool terminator, int voiceTargetID);
 
 		void initializeMixer();
 
@@ -162,6 +147,10 @@ class AudioInput : public QThread {
 		void doDeaf();
 		void doMute();
 	public:
+		typedef enum { ActivityStateIdle, ActivityStateReturnedFromIdle, ActivityStateActive } ActivityState;
+
+		ActivityState activityState;
+
 		bool bResetProcessor;
 
 		Timer tIdle;
@@ -173,9 +162,18 @@ class AudioInput : public QThread {
 		static int getNetworkBandwidth(int bitrate, int frames);
 		static void setMaxBandwidth(int bitspersec);
 
+		/// Construct an AudioInput.
+		///
+		/// This constructor is only ever called by Audio::startInput(), and is guaranteed
+		/// to be called on the application's main thread.
 		AudioInput();
-		~AudioInput();
-		void run() = 0;
+
+		/// Destroy an AudioInput.
+		///
+		/// This destructor is only ever called by Audio::stopInput() and Audio::stop(),
+		/// and is guaranteed to be called on the application's main thread.
+		~AudioInput() Q_DECL_OVERRIDE;
+		void run() Q_DECL_OVERRIDE = 0;
 		virtual bool isAlive() const;
 		bool isTransmitting() const;
 };
