@@ -1,35 +1,17 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "mumble_pch.hpp"
 
+#define DIRECTSOUND_VERSION 0x1000
+
+#include <mmsystem.h>
+#include <dsound.h>
+#include <ks.h>
+#include <ksmedia.h>
+
 #include "DirectSound.h"
 
 #include "MainWindow.h"
@@ -37,6 +19,9 @@
 #include "User.h"
 #include "Global.h"
 
+// from os_win.cpp
+extern HWND mumble_mw_hwnd;
+
 #undef FAILED
 #define FAILED(Status) (static_cast<HRESULT>(Status)<0)
 
@@ -223,7 +208,7 @@ void DXAudioOutput::run() {
 	LPDIRECTSOUND8             pDS = NULL;
 	LPDIRECTSOUNDBUFFER       pDSBPrimary = NULL;
 	LPDIRECTSOUNDBUFFER       pDSBOutput = NULL;
-	LPDIRECTSOUNDNOTIFY8       pDSNotify = NULL;
+	LPDIRECTSOUNDNOTIFY       pDSNotify = NULL;
 
 	int iLastwriteblock;
 	LPVOID aptr1, aptr2;
@@ -258,7 +243,7 @@ void DXAudioOutput::run() {
 	if (! pDS && FAILED(hr = DirectSoundCreate8(&DSDEVID_DefaultVoicePlayback, &pDS, NULL))) {
 		qWarning("DXAudioOutput: DirectSoundCreate failed: hr=0x%08lx", hr);
 		goto cleanup;
-	} else if (FAILED(hr = pDS->SetCooperativeLevel(g.mw->winId(), DSSCL_PRIORITY))) {
+	} else if (FAILED(hr = pDS->SetCooperativeLevel(mumble_mw_hwnd, DSSCL_PRIORITY))) {
 		qWarning("DXAudioOutput: SetCooperativeLevel failed: hr=0x%08lx", hr);
 		goto cleanup;
 	} else if (FAILED(hr = pDS->CreateSoundBuffer(&dsbdesc, &pDSBPrimary, NULL))) {
@@ -479,7 +464,7 @@ DXAudioInput::~DXAudioInput() {
 void DXAudioInput::run() {
 	LPDIRECTSOUNDCAPTURE8      pDSCapture;
 	LPDIRECTSOUNDCAPTUREBUFFER pDSCaptureBuffer;
-	LPDIRECTSOUNDNOTIFY8       pDSNotify;
+	LPDIRECTSOUNDNOTIFY        pDSNotify;
 
 	DWORD dwBufferSize;
 	bool bOk;
@@ -567,7 +552,7 @@ void DXAudioInput::run() {
 				else
 					dwReadyBytes = dwReadPosition - dwLastReadPos;
 
-				if (static_cast<int>(dwReadyBytes) < sizeof(short) * iFrameSize) {
+				if (static_cast<size_t>(dwReadyBytes) < sizeof(short) * iFrameSize) {
 					double msecleft = 20.0 - (dwReadyBytes * 20.0) / (sizeof(short) * iFrameSize);
 
 					if (didsleep)
@@ -582,7 +567,7 @@ void DXAudioInput::run() {
 					didsleep = true;
 					firstsleep = false;
 				}
-			} while (static_cast<int>(dwReadyBytes) < sizeof(short) * iFrameSize);
+			} while (static_cast<size_t>(dwReadyBytes) < sizeof(short) * iFrameSize);
 
 			// Desynchonized?
 			if (dwReadyBytes > (dwBufferSize / 2)) {
@@ -590,7 +575,7 @@ void DXAudioInput::run() {
 				dwLastReadPos = dwReadPosition;
 			} else if (bRunning) {
 				if (FAILED(hr = pDSCaptureBuffer->Lock(dwLastReadPos, sizeof(short) * iFrameSize, &aptr1, &nbytes1, &aptr2, &nbytes2, 0))) {
-					qWarning("DXAudioInput: Lock from %ld (%d bytes) failed: hr=0x%08lx",dwLastReadPos, sizeof(short) * iFrameSize, hr);
+					qWarning("DXAudioInput: Lock from %lu (%zu bytes) failed: hr=0x%08lx", static_cast<unsigned long>(dwLastReadPos), sizeof(short) * iFrameSize, hr);
 					bRunning = false;
 					break;
 				}
