@@ -1,36 +1,10 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
-
-#ifndef GLOBAL_H_
-#define GLOBAL_H_
+#ifndef MUMBLE_MUMBLE_GLOBAL_H_
+#define MUMBLE_MUMBLE_GLOBAL_H_
 
 #include <boost/shared_ptr.hpp>
 #include <QtCore/QDir>
@@ -55,10 +29,13 @@ class LCD;
 class BonjourClient;
 class OverlayClient;
 class CELTCodec;
+class OpusCodec;
+class LogEmitter;
+class DeveloperConsole;
 
 class QNetworkAccessManager;
 
-struct Global {
+struct Global Q_DECL_FINAL {
 private:
 	Q_DISABLE_COPY(Global)
 public:
@@ -68,6 +45,9 @@ public:
 	boost::shared_ptr<ServerHandler> sh;
 	boost::shared_ptr<AudioInput> ai;
 	boost::shared_ptr<AudioOutput> ao;
+	/**
+	 * @remark Must only be accessed from the main event loop
+	 */
 	Database *db;
 	Log *l;
 	Plugins *p;
@@ -76,10 +56,16 @@ public:
 	LCD *lcd;
 	BonjourClient *bc;
 	QNetworkAccessManager *nam;
+	QSharedPointer<LogEmitter> le;
+	DeveloperConsole *c;
 	int iPushToTalk;
 	Timer tDoublePush;
 	quint64 uiDoublePush;
+	/// Holds the current VoiceTarget ID to send audio to
 	int iTarget;
+	/// Holds the value of iTarget before its last change until the current
+	/// audio-stream ends (and it has a value > 0). See the comment in
+	/// AudioInput::flushCheck for further details on this.
 	int iPrevTarget;
 	bool bPushToMute;
 	bool bCenterPosition;
@@ -93,15 +79,19 @@ public:
 	int iAudioBandwidth;
 	QDir qdBasePath;
 	QMap<int, CELTCodec *> qmCodecs;
+	OpusCodec *oCodec;
 	int iCodecAlpha, iCodecBeta;
 	bool bPreferAlpha;
 	bool bOpus;
 	bool bAttenuateOthers;
+	/// If set the AudioOutput::mix will forcefully adjust the volume of all
+	/// non-priority speakers.
+	bool prioritySpeakerActiveOverride;
 	bool bAllowHTML;
 	unsigned int uiMessageLength;
 	unsigned int uiImageLength;
+	unsigned int uiMaxUsers;
 	bool bQuit;
-	QString qsCurrentStyle;
 
 	bool bHappyEaster;
 	static const char ccHappyEaster[];
@@ -135,6 +125,9 @@ class DeferInit {
 		static void run_destroyers();
 };
 
+/// Special exit code which causes mumble to restart itself. The outward facing return code with be 0
+const int MUMBLE_EXIT_CODE_RESTART = 64738;
+
 // -Wshadow is bugged. If an inline function of a class uses a variable or
 // parameter named 'g', that will generate a warning even if the class header
 // is included long before this definition.
