@@ -1,33 +1,7 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "mumble_pch.hpp"
 
@@ -36,13 +10,15 @@
 #include "AudioOutput.h"
 #include "AudioOutputSample.h"
 #include "Channel.h"
-#include "Global.h"
 #include "MainWindow.h"
 #include "NetworkConfig.h"
 #include "RichTextEditor.h"
 #include "ServerHandler.h"
 #include "TextToSpeech.h"
 
+// We define a global macro called 'g'. This can lead to issues when included code uses 'g' as a type or parameter name (like protobuf 3.7 does). As such, for now, we have to make this our last include.
+#include "Global.h"
+
 static ConfigWidget *LogConfigDialogNew(Settings &st) {
 	return new LogConfig(st);
 }
@@ -52,37 +28,55 @@ static ConfigRegistrar registrar(4000, LogConfigDialogNew);
 LogConfig::LogConfig(Settings &st) : ConfigWidget(st) {
 	setupUi(this);
 
+#ifdef USE_NO_TTS
+	qgbTTS->setDisabled(true);
+#endif
 
+#if QT_VERSION >= 0x050000
+	qtwMessages->header()->setSectionResizeMode(ColMessage, QHeaderView::Stretch);
+	qtwMessages->header()->setSectionResizeMode(ColConsole, QHeaderView::ResizeToContents);
+	qtwMessages->header()->setSectionResizeMode(ColNotification, QHeaderView::ResizeToContents);
+	qtwMessages->header()->setSectionResizeMode(ColHighlight, QHeaderView::ResizeToContents);
+	qtwMessages->header()->setSectionResizeMode(ColTTS, QHeaderView::ResizeToContents);
+	qtwMessages->header()->setSectionResizeMode(ColStaticSound, QHeaderView::ResizeToContents);
+#else
 	qtwMessages->header()->setResizeMode(ColMessage, QHeaderView::Stretch);
 	qtwMessages->header()->setResizeMode(ColConsole, QHeaderView::ResizeToContents);
 	qtwMessages->header()->setResizeMode(ColNotification, QHeaderView::ResizeToContents);
+	qtwMessages->header()->setResizeMode(ColHighlight, QHeaderView::ResizeToContents);
 	qtwMessages->header()->setResizeMode(ColTTS, QHeaderView::ResizeToContents);
 	qtwMessages->header()->setResizeMode(ColStaticSound, QHeaderView::ResizeToContents);
+#endif
 
 	QTreeWidgetItem *twi;
 	for (int i = Log::firstMsgType; i <= Log::lastMsgType; ++i) {
-		Log::MsgType t = static_cast<Log::MsgType>(i);
+		Log::MsgType t = Log::msgOrder[i];
 		const QString messageName = g.l->msgName(t);
 
 		twi = new QTreeWidgetItem(qtwMessages);
-		twi->setData(ColMessage, Qt::UserRole, i);
+		twi->setData(ColMessage, Qt::UserRole, static_cast<int>(t));
 		twi->setText(ColMessage, messageName);
 		twi->setCheckState(ColConsole, Qt::Unchecked);
 		twi->setCheckState(ColNotification, Qt::Unchecked);
-		twi->setCheckState(ColTTS, Qt::Unchecked);
+		twi->setCheckState(ColHighlight, Qt::Unchecked);
 		twi->setCheckState(ColStaticSound, Qt::Unchecked);
 
 		twi->setToolTip(ColConsole, tr("Toggle console for %1 events").arg(messageName));
 		twi->setToolTip(ColNotification, tr("Toggle pop-up notifications for %1 events").arg(messageName));
-		twi->setToolTip(ColTTS, tr("Toggle Text-To-Speech for %1 events").arg(messageName));
+		twi->setToolTip(ColHighlight, tr("Toggle window highlight (if not active) for %1 events").arg(messageName));
 		twi->setToolTip(ColStaticSound, tr("Click here to toggle sound notification for %1 events").arg(messageName));
 		twi->setToolTip(ColStaticSoundPath, tr("Path to sound file used for sound notifications in the case of %1 events<br />Single click to play<br />Double-click to change").arg(messageName));
 
 		twi->setWhatsThis(ColConsole, tr("Click here to toggle console output for %1 events.<br />If checked, this option makes Mumble output all %1 events in its message log.").arg(messageName));
 		twi->setWhatsThis(ColNotification, tr("Click here to toggle pop-up notifications for %1 events.<br />If checked, a notification pop-up will be created by Mumble for every %1 event.").arg(messageName));
-		twi->setWhatsThis(ColTTS, tr("Click here to toggle Text-To-Speech for %1 events.<br />If checked, Mumble uses Text-To-Speech to read %1 events out loud to you. Text-To-Speech is also able to read the contents of the event which is not true for sound files. Text-To-Speech and sound files cannot be used at the same time.").arg(messageName));
+		twi->setWhatsThis(ColHighlight, tr("Click here to toggle window highlight for %1 events.<br />If checked, Mumble's window will be highlighted for every %1 event, if not active.").arg(messageName));
 		twi->setWhatsThis(ColStaticSound, tr("Click here to toggle sound notification for %1 events.<br />If checked, Mumble uses a sound file predefined by you to indicate %1 events. Sound files and Text-To-Speech cannot be used at the same time.").arg(messageName));
 		twi->setWhatsThis(ColStaticSoundPath, tr("Path to sound file used for sound notifications in the case of %1 events.<br />Single click to play<br />Double-click to change<br />Ensure that sound notifications for these events are enabled or this field will not have any effect.").arg(messageName));
+#ifndef USE_NO_TTS
+		twi->setCheckState(ColTTS, Qt::Unchecked);
+		twi->setToolTip(ColTTS, tr("Toggle Text-To-Speech for %1 events").arg(messageName));
+		twi->setWhatsThis(ColTTS, tr("Click here to toggle Text-To-Speech for %1 events.<br />If checked, Mumble uses Text-To-Speech to read %1 events out loud to you. Text-To-Speech is also able to read the contents of the event which is not true for sound files. Text-To-Speech and sound files cannot be used at the same time.").arg(messageName));
+#endif
 	}
 }
 
@@ -96,21 +90,30 @@ QIcon LogConfig::icon() const {
 
 void LogConfig::load(const Settings &r) {
 	QList<QTreeWidgetItem *> qlItems = qtwMessages->findItems(QString(), Qt::MatchContains);
+
 	foreach(QTreeWidgetItem *i, qlItems) {
 		Log::MsgType mt = static_cast<Log::MsgType>(i->data(ColMessage, Qt::UserRole).toInt());
 		Settings::MessageLog ml = static_cast<Settings::MessageLog>(r.qmMessages.value(mt));
 
 		i->setCheckState(ColConsole, (ml & Settings::LogConsole) ? Qt::Checked : Qt::Unchecked);
 		i->setCheckState(ColNotification, (ml & Settings::LogBalloon) ? Qt::Checked : Qt::Unchecked);
+		i->setCheckState(ColHighlight, (ml & Settings::LogHighlight) ? Qt::Checked : Qt::Unchecked);
+#ifndef USE_NO_TTS
 		i->setCheckState(ColTTS, (ml & Settings::LogTTS) ? Qt::Checked : Qt::Unchecked);
+#endif
 		i->setCheckState(ColStaticSound, (ml & Settings::LogSoundfile) ? Qt::Checked : Qt::Unchecked);
 		i->setText(ColStaticSoundPath, r.qmMessageSounds.value(mt));
 	}
+
 	qsbMaxBlocks->setValue(r.iMaxLogBlocks);
 
+#ifdef USE_NO_TTS
+	qtwMessages->hideColumn(ColTTS);
+#else
 	loadSlider(qsVolume, r.iTTSVolume);
 	qsbThreshold->setValue(r.iTTSThreshold);
 	qcbReadBackOwn->setChecked(r.bTTSMessageReadBack);
+#endif
 	qcbWhisperFriends->setChecked(r.bWhisperFriends);
 }
 
@@ -124,8 +127,12 @@ void LogConfig::save() const {
 			v |= Settings::LogConsole;
 		if (i->checkState(ColNotification) == Qt::Checked)
 			v |= Settings::LogBalloon;
+		if (i->checkState(ColHighlight) == Qt::Checked)
+			v |= Settings::LogHighlight;
+#ifndef USE_NO_TTS
 		if (i->checkState(ColTTS) == Qt::Checked)
 			v |= Settings::LogTTS;
+#endif
 		if (i->checkState(ColStaticSound) == Qt::Checked)
 			v |= Settings::LogSoundfile;
 		s.qmMessages[mt] = v;
@@ -133,9 +140,11 @@ void LogConfig::save() const {
 	}
 	s.iMaxLogBlocks = qsbMaxBlocks->value();
 
+#ifndef USE_NO_TTS
 	s.iTTSVolume=qsVolume->value();
 	s.iTTSThreshold=qsbThreshold->value();
 	s.bTTSMessageReadBack = qcbReadBackOwn->isChecked();
+#endif
 	s.bWhisperFriends = qcbWhisperFriends->isChecked();
 }
 
@@ -144,10 +153,6 @@ void LogConfig::accept() const {
 	g.mw->qteLog->document()->setMaximumBlockCount(s.iMaxLogBlocks);
 }
 
-bool LogConfig::expert(bool) {
-	return false;
-}
-
 void LogConfig::on_qtwMessages_itemChanged(QTreeWidgetItem* i, int column) {
 	if (! i->isSelected()) return;
 	switch (column) {
@@ -182,9 +187,10 @@ void LogConfig::on_qtwMessages_itemDoubleClicked(QTreeWidgetItem * item, int col
 }
 
 void LogConfig::browseForAudioFile() {
-	QString file = AudioOutputSample::browseForSndfile();
+	QTreeWidgetItem *i = qtwMessages->selectedItems()[0];
+	QString defaultpath(i->text(ColStaticSoundPath));
+	QString file = AudioOutputSample::browseForSndfile(defaultpath);
 	if (!file.isEmpty()) {
-		QTreeWidgetItem *i = qtwMessages->selectedItems()[0];
 		i->setText(ColStaticSoundPath, file);
 		i->setCheckState(ColStaticSound, Qt::Checked);
 	}
@@ -197,6 +203,23 @@ Log::Log(QObject *p) : QObject(p) {
 	qdDate = QDate::currentDate();
 }
 
+// Display order in settingsscreen, allows to insert new events without breaking config-compatibility with older versions
+const Log::MsgType Log::msgOrder[] = {
+	DebugInfo, CriticalError, Warning, Information,
+	ServerConnected, ServerDisconnected,
+	UserJoin, UserLeave,
+	Recording,
+	YouKicked, UserKicked,
+	UserRenamed,
+	SelfMute, SelfUnmute, SelfDeaf, SelfUndeaf,
+	OtherSelfMute, YouMuted, YouMutedOther, OtherMutedOther,
+	SelfChannelJoin, SelfChannelJoinOther,
+	ChannelJoin, ChannelLeave,
+	ChannelJoinConnect, ChannelLeaveDisconnect,
+	PermissionDenied,
+	TextMessage, PrivateTextMessage
+};
+
 const char *Log::msgNames[] = {
 	QT_TRANSLATE_NOOP("Log", "Debug"),
 	QT_TRANSLATE_NOOP("Log", "Critical"),
@@ -209,7 +232,7 @@ const char *Log::msgNames[] = {
 	QT_TRANSLATE_NOOP("Log", "User recording state changed"),
 	QT_TRANSLATE_NOOP("Log", "User kicked (you or by you)"),
 	QT_TRANSLATE_NOOP("Log", "User kicked"),
-	QT_TRANSLATE_NOOP("Log", "You self-muted/deafened"),
+	QT_TRANSLATE_NOOP("Log", "You self-muted"),
 	QT_TRANSLATE_NOOP("Log", "Other self-muted/deafened"),
 	QT_TRANSLATE_NOOP("Log", "User muted (you)"),
 	QT_TRANSLATE_NOOP("Log", "User muted (by you)"),
@@ -217,7 +240,16 @@ const char *Log::msgNames[] = {
 	QT_TRANSLATE_NOOP("Log", "User Joined Channel"),
 	QT_TRANSLATE_NOOP("Log", "User Left Channel"),
 	QT_TRANSLATE_NOOP("Log", "Permission Denied"),
-	QT_TRANSLATE_NOOP("Log", "Text Message")
+	QT_TRANSLATE_NOOP("Log", "Text Message"),
+	QT_TRANSLATE_NOOP("Log", "You self-unmuted"),
+	QT_TRANSLATE_NOOP("Log", "You self-deafened"),
+	QT_TRANSLATE_NOOP("Log", "You self-undeafened"),
+	QT_TRANSLATE_NOOP("Log", "User renamed"),
+	QT_TRANSLATE_NOOP("Log", "You Joined Channel"),
+	QT_TRANSLATE_NOOP("Log", "You Joined Channel (moved)"),
+	QT_TRANSLATE_NOOP("Log", "User connected and entered channel"),
+	QT_TRANSLATE_NOOP("Log", "User left channel and disconnected"),
+	QT_TRANSLATE_NOOP("Log", "Private text message")
 };
 
 QString Log::msgName(MsgType t) const {
@@ -245,6 +277,7 @@ const QStringList Log::allowedSchemes() {
 	qslAllowedSchemeNames << QLatin1String("mailto");
 	qslAllowedSchemeNames << QLatin1String("xmpp");
 	qslAllowedSchemeNames << QLatin1String("skype");
+	qslAllowedSchemeNames << QLatin1String("rtmp"); // http://en.wikipedia.org/wiki/Real_Time_Messaging_Protocol
 
 	return qslAllowedSchemeNames;
 }
@@ -259,7 +292,7 @@ QString Log::formatChannel(::Channel *c) {
 	return QString::fromLatin1("<a href='channelid://%1/%3' class='log-channel'>%2</a>").arg(c->iId).arg(Qt::escape(c->qsName)).arg(QString::fromLatin1(g.sh->qbaDigest.toBase64()));
 }
 
-QString Log::formatClientUser(ClientUser *cu, LogColorType t) {
+QString Log::formatClientUser(ClientUser *cu, LogColorType t, const QString &displayName) {
 	QString className;
 	if (t == Log::Target) {
 		className = QString::fromLatin1("target");
@@ -268,7 +301,7 @@ QString Log::formatClientUser(ClientUser *cu, LogColorType t) {
 	}
 
 	if (cu) {
-		QString name = Qt::escape(cu->qsName);
+		QString name = Qt::escape(displayName.isNull() ? cu->qsName : displayName);
 		if (cu->qsHash.isEmpty()) {
 			return QString::fromLatin1("<a href='clientid://%2/%4' class='log-user log-%1'>%3</a>").arg(className).arg(cu->uiSession).arg(name).arg(QString::fromLatin1(g.sh->qbaDigest.toBase64()));
 		} else {
@@ -346,13 +379,9 @@ QString Log::imageToImg(QImage img) {
 	return QString();
 }
 
-QString Log::validHtml(const QString &html, bool allowReplacement, QTextCursor *tc) {
+QString Log::validHtml(const QString &html, QTextCursor *tc) {
 	QDesktopWidget dw;
 	LogDocument qtd;
-	bool valid = false;
-
-	qtd.setAllowHTTPResources(allowReplacement);
-	qtd.setOnlyLoadDataURLs(true);
 
 	QRectF qr = dw.availableGeometry(dw.screenNumber(g.mw));
 	qtd.setTextWidth(qr.width() / 2);
@@ -366,7 +395,6 @@ QString Log::validHtml(const QString &html, bool allowReplacement, QTextCursor *
 	// data URL images to run.
 	(void) qtd.documentLayout();
 	qtd.setHtml(html);
-	valid = qtd.isValid();
 
 	QStringList qslAllowed = allowedSchemes();
 	for (QTextBlock qtb = qtd.begin(); qtb != qtd.end(); qtb = qtb.next()) {
@@ -384,31 +412,32 @@ QString Log::validHtml(const QString &html, bool allowReplacement, QTextCursor *
 					qtbi = qtb.begin();
 				}
 			}
-			if (qcf.isImageFormat()) {
-				QTextImageFormat qtif = qcf.toImageFormat();
-				QUrl url(qtif.name());
-				if (! qtif.name().isEmpty() && ! url.isValid())
-					valid = false;
-			}
 		}
 	}
 
 	qtd.adjustSize();
 	QSizeF s = qtd.size();
 
-	if (!valid || !s.isValid() || (s.width() > qr.width()) || (s.height() > qr.height())) {
-		qtd.setPlainText(html);
-		qtd.adjustSize();
-		s = qtd.size();
+	if (!s.isValid()) {
+		QString errorInvalidSizeMessage = tr("[[ Invalid size ]]");
+		if (tc) {
+			tc->insertText(errorInvalidSizeMessage);
+			return QString();
+		} else {
+			return errorInvalidSizeMessage;
+		}
+	}
 
-		if (!s.isValid() || (s.width() > qr.width()) || (s.height() > qr.height())) {
-			QString errorMessage = tr("[[ Text object too large to display ]]");
-			if (tc) {
-				tc->insertText(errorMessage);
-				return QString();
-			} else {
-				return errorMessage;
-			}
+	int messageSize = s.width() * s.height();
+	int allowedSize = 2048 * 2048;
+
+	if (messageSize > allowedSize) {
+		QString errorSizeMessage = tr("[[ Text object too large to display ]]");
+		if (tc) {
+			tc->insertText(errorSizeMessage);
+			return QString();
+		} else {
+			return errorSizeMessage;
 		}
 	}
 
@@ -441,6 +470,8 @@ void Log::log(MsgType mt, const QString &console, const QString &terse, bool own
 		QTextCursor tc = g.mw->qteLog->textCursor();
 
 		LogTextBrowser *tlog = g.mw->qteLog;
+		const int oldscrollvalue = tlog->getLogScroll();
+		const bool scroll = (oldscrollvalue == tlog->getLogScrollMaximum());
 
 		tc.movePosition(QTextCursor::End);
 
@@ -460,41 +491,55 @@ void Log::log(MsgType mt, const QString &console, const QString &terse, bool own
 		} else if (! g.mw->qteLog->document()->isEmpty()) {
 			tc.insertBlock();
 		}
-		tc.insertHtml(Log::msgColor(QString::fromLatin1("[%1] ").arg(Qt::escape(dt.time().toString(Qt::DefaultLocaleShortDate))), Log::Time));
-		validHtml(console, true, &tc);
+		tc.insertHtml(Log::msgColor(QString::fromLatin1("[%1] ").arg(Qt::escape(dt.time().toString())), Log::Time));
+		validHtml(console, &tc);
 		tc.movePosition(QTextCursor::End);
 		g.mw->qteLog->setTextCursor(tc);
 
-		tlog->updateScrollPosition();
+		if (scroll || ownMessage)
+			tlog->scrollLogToBottom();
+		else
+			tlog->setLogScroll(oldscrollvalue);
 	}
 
-	if (!g.s.bTTSMessageReadBack && ownMessage)
-		return;
+	if (!ownMessage) {
+		if (!(g.mw->isActiveWindow() && g.mw->qdwLog->isVisible())) {
+			// Message notification with window highlight
+			if (flags & Settings::LogHighlight) {
+				QApplication::alert(g.mw);
+			}
 
-	// Message notification with balloon tooltips
-	if ((flags & Settings::LogBalloon) && !(g.mw->isActiveWindow() && g.mw->qdwLog->isVisible()))
-		postNotification(mt, console, plain);
-
-	// Don't make any noise if we are self deafened
-	if (g.s.bDeaf)
-		return;
-
-	// Message notification with static sounds
-	if ((flags & Settings::LogSoundfile)) {
-		QString sSound = g.s.qmMessageSounds.value(mt);
-		AudioOutputPtr ao = g.ao;
-		if (!ao || !ao->playSample(sSound, false)) {
-			qWarning() << "Sound file" << sSound << "is not a valid audio file, fallback to TTS.";
-			flags ^= Settings::LogSoundfile | Settings::LogTTS; // Fallback to TTS
+			// Message notification with balloon tooltips
+			if (flags & Settings::LogBalloon) {
+				postNotification(mt, plain);
+			}
 		}
+
+		// Don't make any noise if we are self deafened (Unless it is the sound for activating self deaf)
+		if (g.s.bDeaf && mt != Log::SelfDeaf) {
+			return;
+		}
+
+		// Message notification with static sounds
+		if ((flags & Settings::LogSoundfile)) {
+			QString sSound = g.s.qmMessageSounds.value(mt);
+			AudioOutputPtr ao = g.ao;
+			if (!ao || !ao->playSample(sSound, false)) {
+				qWarning() << "Sound file" << sSound << "is not a valid audio file, fallback to TTS.";
+				flags ^= Settings::LogSoundfile | Settings::LogTTS; // Fallback to TTS
+			}
+		}
+	} else if (!g.s.bTTSMessageReadBack) {
+		return;
 	}
 
 	// Message notification with Text-To-Speech
-	if (! g.s.bTTS || !(flags & Settings::LogTTS))
+	if (g.s.bDeaf || !g.s.bTTS || !(flags & Settings::LogTTS)) {
 		return;
+	}
 
 	// Apply simplifications to spoken text
-	QRegExp identifyURL(QLatin1String("[a-z-]+://[^ <$]*"),
+	QRegExp identifyURL(QLatin1String("[a-z-]+://[^ <]*"),
 	                    Qt::CaseInsensitive,
 	                    QRegExp::RegExp2);
 
@@ -512,7 +557,7 @@ void Log::log(MsgType mt, const QString &console, const QString &terse, bool own
 			if (url.scheme() == QLatin1String("http") || url.scheme() == QLatin1String("https"))
 				replacement = tr("link to %1").arg(host);
 			else if (url.scheme() == QLatin1String("ftp"))
-				replacement = tr("ftp link to %1").arg(host);
+				replacement = tr("FTP link to %1").arg(host);
 			else if (url.scheme() == QLatin1String("clientid"))
 				replacement = tr("player link");
 			else if (url.scheme() == QLatin1String("channelid"))
@@ -554,89 +599,39 @@ void Log::postQtNotification(MsgType mt, const QString &plain) {
 }
 
 LogDocument::LogDocument(QObject *p)
-	: QTextDocument(p)
-	, m_valid(true)
-	, m_onlyLoadDataURLs(false)
-	, m_allowHTTPResources(true) {
-	connect(documentLayout(), SIGNAL(update(const QRectF &)), SLOT(layoutUpdate()));
-	connect(documentLayout(), SIGNAL(documentSizeChanged(const QSizeF &)), SLOT(layoutUpdate()));
+	: QTextDocument(p) {
 }
 
 QVariant LogDocument::loadResource(int type, const QUrl &url) {
+	// Ignore requests for all external resources
+	// that aren't images. We don't support any of them.
 	if (type != QTextDocument::ImageResource) {
-		m_valid = false;
-		return QLatin1String("No external resources allowed.");
-	}
-
-	if (url.scheme() != QLatin1String("data") && g.s.iMaxImageSize <= 0) {
-		m_valid = false;
-		return QLatin1String("Image download disabled.");
+		addResource(type, url, QByteArray());
+		return QByteArray();
 	}
 
 	QImage qi(1, 1, QImage::Format_Mono);
 	addResource(type, url, qi);
 
-	if (! url.isValid() || url.isRelative()) {
-		m_valid = false;
+	if (! url.isValid()) {
 		return qi;
 	}
 
-	QStringList allowedSchemes;
-	allowedSchemes << QLatin1String("data");
-	if (m_allowHTTPResources) {
-		allowedSchemes << QLatin1String("http");
-		allowedSchemes << QLatin1String("https");
-	}
-
-	if (!allowedSchemes.contains(url.scheme())) {
-		m_valid = false;
+	if (url.scheme() != QLatin1String("data")) {
 		return qi;
 	}
 
-	bool shouldLoad = true;
-	if (m_onlyLoadDataURLs && url.scheme() != QLatin1String("data")) {
-		shouldLoad = false;
-	}
+	QNetworkReply *rep = Network::get(url);
+	connect(rep, SIGNAL(finished()), this, SLOT(finished()));
 
-	if (shouldLoad) {
-		QNetworkReply *rep = Network::get(url);
-		connect(rep, SIGNAL(metaDataChanged()), this, SLOT(receivedHead()));
-		connect(rep, SIGNAL(finished()), this, SLOT(finished()));
-
-		// Handle data URLs immediately without a roundtrip to the event loop.
-		// We need this to perform proper validation for data URL images when
-		// a LogDocument is used inside Log::validHtml().
-		if (url.scheme() == QLatin1String("data")) {
-			QCoreApplication::sendPostedEvents(rep, 0);
-		}
-	}
+	// Handle data URLs immediately without a roundtrip to the event loop.
+	// We need this to perform proper validation for data URL images when
+	// a LogDocument is used inside Log::validHtml().
+	QCoreApplication::sendPostedEvents(rep, 0);
 
 	return qi;
 }
 
-void LogDocument::setAllowHTTPResources(bool allowHTTPResources) {
-	m_allowHTTPResources = allowHTTPResources;
-}
-
-void LogDocument::setOnlyLoadDataURLs(bool onlyLoadDataURLs) {
-	m_onlyLoadDataURLs = onlyLoadDataURLs;
-}
-
-bool LogDocument::isValid() {
-	return m_valid;
-}
-
-void LogDocument::receivedHead() {
-	QNetworkReply *rep = qobject_cast<QNetworkReply *>(sender());
-	if (rep->url().scheme() != QLatin1String("data")) {
-		QVariant length = rep->header(QNetworkRequest::ContentLengthHeader);
-		if (length == QVariant::Invalid || length.toInt() > g.s.iMaxImageSize) {
-			m_valid = false;
-			rep->abort();
-		}
-	}
-}
-
 void LogDocument::finished() {
 	QNetworkReply *rep = qobject_cast<QNetworkReply *>(sender());
 
@@ -651,47 +646,29 @@ void LogDocument::finished() {
 		// instead of strictly requiring a correct Content-Type.
 		if (RichTextImage::isValidImage(ba, fmt)) {
 			if (qi.loadFromData(ba, fmt)) {
-				bool ok = true;
-				if (rep->url().scheme() != QLatin1String("data")) {
-					ok = (qi.width() <= g.s.iMaxImageWidth && qi.height() <= g.s.iMaxImageHeight);
-				}
-				if (ok) {
-					addResource(QTextDocument::ImageResource, rep->request().url(), qi);
+				addResource(QTextDocument::ImageResource, rep->request().url(), qi);
 
-					// Force a re-layout of the QTextEdit the next
-					// time we enter the event loop.
-					// We must not trigger a re-layout immediately.
-					// Doing so can trigger crashes deep inside Qt
-					// if the QTextDocument has just been set on the
-					// text edit widget.
-					QTextEdit *qte = qobject_cast<QTextEdit *>(parent());
-					if (qte != NULL) {
-						QEvent *e = new QEvent(QEvent::FontChange);
-						QApplication::postEvent(qte, e);
+				// Force a re-layout of the QTextEdit the next
+				// time we enter the event loop.
+				// We must not trigger a re-layout immediately.
+				// Doing so can trigger crashes deep inside Qt
+				// if the QTextDocument has just been set on the
+				// text edit widget.
+				QTextEdit *qte = qobject_cast<QTextEdit *>(parent());
+				if (qte != NULL) {
+					QEvent *e = new QEvent(QEvent::FontChange);
+					QApplication::postEvent(qte, e);
 
-						e = new LogDocumentResourceAddedEvent();
-						QApplication::postEvent(qte, e);
-					}
-				} else {
-					m_valid = false;
+					e = new LogDocumentResourceAddedEvent();
+					QApplication::postEvent(qte, e);
 				}
 			}
-		} else {
-			m_valid = false;
 		}
 	}
 
 	rep->deleteLater();
 }
 
-void LogDocument::layoutUpdate() {
-	QTextEdit *qte = qobject_cast<QTextEdit *>(parent());
-	if (qte != NULL) {
-		QEvent *e = new LogDocumentResourceAddedEvent();
-		QApplication::postEvent(qte, e);
-	}
-}
-
 LogDocumentResourceAddedEvent::LogDocumentResourceAddedEvent()
 	: QEvent(LogDocumentResourceAddedEvent::Type) {
 }
