@@ -199,13 +199,25 @@ Overlay::~Overlay() {
 	}
 }
 
-void Overlay::setActive(bool act) {
+void Overlay::initialize() {
 	if (!m_initialized) {
 		platformInit();
 		forceSettings();
+
+		createPipe();
 	
 		m_initialized = true;
 	}
+}
+
+void Overlay::setActive(bool act) {
+	if (!act && !m_initialized) {
+		// Disabling when the Overlay hasn't been initialized yet, doesn't make much sense
+		return;
+	}
+
+	// Make sure the Overlay is initialized
+	initialize();
 
 	setActiveInternal(act);
 }
