@@ -1,50 +1,34 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "mumble_pch.hpp"
 
-#include "Overlay.h"
+#include "OverlayClient.h"
+#include "OverlayPositionableItem.h"
+#include "OverlayEditor.h"
 #include "OverlayText.h"
 #include "User.h"
 #include "Channel.h"
-#include "Global.h"
 #include "Message.h"
 #include "Database.h"
 #include "NetworkConfig.h"
 #include "ServerHandler.h"
 #include "MainWindow.h"
 #include "GlobalShortcut.h"
+#include "Themes.h"
 
-OverlayClient::OverlayClient(QLocalSocket *socket, QObject *p) :
-		QObject(p),
-		ougUsers(&g.s.os) {
+// We define a global macro called 'g'. This can lead to issues when included code uses 'g' as a type or parameter name (like protobuf 3.7 does). As such, for now, we have to make this our last include.
+#include "Global.h"
+
+OverlayClient::OverlayClient(QLocalSocket *socket, QObject *p)
+	: QObject(p)
+	, framesPerSecond(0)
+	, ougUsers(&g.s.os)
+	, iMouseX(0)
+	, iMouseY(0) {
+	
 	qlsSocket = socket;
 	qlsSocket->setParent(NULL);
 	connect(qlsSocket, SIGNAL(readyRead()), this, SLOT(readyRead()));
@@ -73,11 +57,17 @@ OverlayClient::OverlayClient(QLocalSocket *socket, QObject *p) :
 	qgs.addItem(&ougUsers);
 	ougUsers.show();
 
-	qgpiFPS = new QGraphicsPixmapItem();
+	qgpiFPS = new OverlayPositionableItem(&g.s.os.qrfFps);
 	qgs.addItem(qgpiFPS);
 	qgpiFPS->setPos(g.s.os.qrfFps.x(), g.s.os.qrfFps.y());
 	qgpiFPS->show();
 
+	// Time
+	qgpiTime = new OverlayPositionableItem(&g.s.os.qrfTime);
+	qgs.addItem(qgpiTime);
+	qgpiTime->setPos(g.s.os.qrfTime.x(), g.s.os.qrfTime.y());
+	qgpiTime->show();
+
 	qgpiLogo = NULL;
 
 	iOffsetX = iOffsetY = 0;
@@ -87,11 +77,16 @@ OverlayClient::OverlayClient(QLocalSocket *socket, QObject *p) :
 
 OverlayClient::~OverlayClient() {
 	delete qgpiFPS;
+	delete qgpiTime;
 	delete qgpiCursor;
 	delete qgpiLogo;
 
-	qlsSocket->disconnect();
-	qlsSocket->abort();
+	qlsSocket->disconnectFromServer();
+	if (!qlsSocket->waitForDisconnected(1000)) {
+		qDebug() << "OverlayClient: Failed to cleanly disconnect: " << qlsSocket->errorString();
+		qlsSocket->abort();
+	}
+	
 	qlsSocket->deleteLater();
 
 	ougUsers.reset();
@@ -107,33 +102,56 @@ bool OverlayClient::eventFilter(QObject *o, QEvent *e) {
 
 void OverlayClient::updateFPS() {
 	if (g.s.os.bFps) {
-		const BasepointPixmap &pm = OverlayTextLine(QString(QLatin1String("%1")).arg(iroundf(fFps + 0.5f)), g.s.os.qfFps).createPixmap(g.s.os.qcFps);
+		const BasepointPixmap &pm = OverlayTextLine(
+		            QString(QLatin1String("%1")).arg(iroundf(framesPerSecond + 0.5f)),
+		            g.s.os.qfFps).createPixmap(g.s.os.qcFps);
+		qgpiFPS->setVisible(true);
 		qgpiFPS->setPixmap(pm);
 		// offset to use basepoint
 		//TODO: settings are providing a top left anchor, so shift down by ascent
 		qgpiFPS->setOffset(-pm.qpBasePoint + QPoint(0, pm.iAscent));
+		qgpiFPS->updateRender();
 	} else {
-		qgpiFPS->setPixmap(QPixmap());
+		qgpiFPS->setVisible(false);
 	}
 }
 
-#ifndef QT_MAC_USE_COCOA
+void OverlayClient::updateTime() {
+	if (g.s.os.bTime) {
+		const BasepointPixmap &pm = OverlayTextLine(QString(QLatin1String("%1")).arg(QTime::currentTime().toString()), g.s.os.qfFps).createPixmap(g.s.os.qcFps);
+		qgpiTime->setVisible(true);
+		qgpiTime->setPixmap(pm);
+		qgpiTime->setOffset(-pm.qpBasePoint + QPoint(0, pm.iAscent));
+		qgpiTime->updateRender();
+	} else {
+		qgpiTime->setVisible(false);
+	}
+}
+
+#if !defined(Q_OS_MAC) || (defined(Q_OS_MAC) && defined(USE_MAC_UNIVERSAL))
 void OverlayClient::updateMouse() {
-#ifdef Q_OS_WIN
+#if defined(Q_OS_WIN)
 	QPixmap pm;
 
 	HICON c = ::GetCursor();
+#if QT_VERSION < 0x050000
 	if (c == NULL)
 		c = qgv.viewport()->cursor().handle();
+#endif
 
 	ICONINFO info;
 	ZeroMemory(&info, sizeof(info));
-	if (::GetIconInfo(c, &info)) {
+	if (c != NULL && ::GetIconInfo(c, &info)) {
+#if QT_VERSION >= 0x050000
+		extern QPixmap qt_pixmapFromWinHBITMAP(HBITMAP bitmap, int format = 0);
+#else
+# define qt_pixmapFromWinHBITMAP(bmp) QPixmap::fromWinHBITMAP(bmp)
+#endif
 		if (info.hbmColor) {
-			pm = QPixmap::fromWinHBITMAP(info.hbmColor);
-			pm.setMask(QBitmap(QPixmap::fromWinHBITMAP(info.hbmMask)));
+			pm = qt_pixmapFromWinHBITMAP(info.hbmColor);
+			pm.setMask(QBitmap(qt_pixmapFromWinHBITMAP(info.hbmMask)));
 		} else {
-			QBitmap orig(QPixmap::fromWinHBITMAP(info.hbmMask));
+			QBitmap orig(qt_pixmapFromWinHBITMAP(info.hbmMask));
 			QImage img = orig.toImage();
 
 			int h = img.height() / 2;
@@ -174,7 +192,7 @@ void OverlayClient::updateMouse() {
 }
 #endif
 
-#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
+#if QT_VERSION < 0x050000 && (defined(Q_OS_WIN) || defined(Q_OS_MAC))
 extern bool Q_GUI_EXPORT qt_use_native_dialogs;
 #endif
 
@@ -183,7 +201,7 @@ extern bool Q_GUI_EXPORT qt_use_native_dialogs;
 // that we're about to reparent.
 
 void OverlayClient::showGui() {
-#if defined(QT3_SUPPORT) || defined(Q_WS_WIN)
+#if defined(QT3_SUPPORT) || (defined(Q_OS_WIN) && QT_VERSION < 0x050000)
 	if (QCoreApplication::loopLevel() > 1)
 		return;
 #else
@@ -243,8 +261,8 @@ outer:
 		}
 	}
 
-	QEvent event(QEvent::WindowActivate);
-	qApp->sendEvent(&qgs, &event);
+	QEvent activateEvent(QEvent::WindowActivate);
+	qApp->sendEvent(&qgs, &activateEvent);
 
 	QPoint p = QCursor::pos();
 	iMouseX = qBound<int>(0, p.x(), uiWidth-1);
@@ -273,7 +291,7 @@ outer:
 
 	setupScene(true);
 
-#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
+#if QT_VERSION < 0x050000 && (defined(Q_OS_WIN) || defined(Q_OS_MAC))
 	qt_use_native_dialogs = false;
 #endif
 
@@ -288,7 +306,7 @@ outer:
 }
 
 void OverlayClient::hideGui() {
-#if defined(QT3_SUPPORT) || defined(Q_WS_WIN)
+#if defined(QT3_SUPPORT) || (defined(Q_OS_WIN) && QT_VERSION < 0x050000)
 	if (QCoreApplication::loopLevel() > 1) {
 		QCoreApplication::exit_loop();
 		QMetaObject::invokeMethod(this, "hideGui", Qt::QueuedConnection);
@@ -340,20 +358,13 @@ void OverlayClient::hideGui() {
 #ifdef Q_OS_MAC
 	qApp->setAttribute(Qt::AA_DontUseNativeMenuBar, false);
 	g.mw->setUnifiedTitleAndToolBarOnMac(true);
-	if (! g.qsCurrentStyle.isEmpty()) {
-		qApp->setStyle(g.qsCurrentStyle);
-	} else {
-		// Assume that an empty qsCurrentStyle means "use the aqua theme".
-		// This might not always be the case (for example, the default style
-		// can be changed via the Qt command line argument "-style".
-		qApp->setStyle(QLatin1String("Macintosh (aqua)"));
-	}
+	Themes::apply();
 #endif
 
 	setupScene(false);
 
 	qgv.setAttribute(Qt::WA_WState_Hidden, true);
-#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
+#if QT_VERSION < 0x050000 && (defined(Q_OS_WIN) || defined(Q_OS_MAC))
 	qt_use_native_dialogs = true;
 #endif
 
@@ -375,14 +386,51 @@ void OverlayClient::scheduleDelete() {
 	hideGui();
 }
 
+void OverlayClient::readyReadMsgInit(unsigned int length) {
+	if (length != sizeof(OverlayMsgInit)) {
+		return;
+	}
+
+	OverlayMsgInit *omi = & omMsg.omi;
+
+	uiWidth = omi->uiWidth;
+	uiHeight = omi->uiHeight;
+	qrLast = QRect();
+
+	delete smMem;
+
+	smMem = new SharedMemory2(this, uiWidth * uiHeight * 4);
+	if (! smMem->data()) {
+		qWarning() << "OverlayClient: Failed to create shared memory" << uiWidth << uiHeight;
+		delete smMem;
+		smMem = NULL;
+		return;
+	}
+	QByteArray key = smMem->name().toUtf8();
+	key.append(static_cast<char>(0));
+
+	OverlayMsg om;
+	om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
+	om.omh.uiType = OVERLAY_MSGTYPE_SHMEM;
+	om.omh.iLength = key.length();
+	Q_ASSERT(sizeof(om.oms.a_cName) >= static_cast<size_t>(key.length())); // Name should be auto-generated and short
+	memcpy(om.oms.a_cName, key.constData(), key.length());
+	qlsSocket->write(om.headerbuffer, sizeof(OverlayMsgHeader) + om.omh.iLength);
+
+	setupRender();
+
+	Overlay *o = static_cast<Overlay *>(parent());
+	QTimer::singleShot(0, o, SLOT(updateOverlay()));
+}
+
 void OverlayClient::readyRead() {
-	while (1) {
-		unsigned int ready = qlsSocket->bytesAvailable();
+	while (true) {
+		quint64 ready = static_cast<quint64>(qlsSocket->bytesAvailable());
 
 		if (omMsg.omh.iLength == -1) {
-			if (ready < sizeof(OverlayMsgHeader))
+			if (ready < sizeof(OverlayMsgHeader)) {
 				break;
-			else {
+			} else {
 				qlsSocket->read(omMsg.headerbuffer, sizeof(OverlayMsgHeader));
 				if ((omMsg.omh.uiMagic != OVERLAY_MAGIC_NUMBER) || (omMsg.omh.iLength < 0) || (omMsg.omh.iLength > static_cast<int>(sizeof(OverlayMsgShmem)))) {
 					disconnect();
@@ -393,7 +441,7 @@ void OverlayClient::readyRead() {
 		}
 
 		if (ready >= static_cast<unsigned int>(omMsg.omh.iLength)) {
-			int length = qlsSocket->read(omMsg.msgbuffer, omMsg.omh.iLength);
+			qint64 length = qlsSocket->read(omMsg.msgbuffer, omMsg.omh.iLength);
 
 			if (length != omMsg.omh.iLength) {
 				disconnect();
@@ -402,38 +450,7 @@ void OverlayClient::readyRead() {
 
 			switch (omMsg.omh.uiType) {
 				case OVERLAY_MSGTYPE_INIT: {
-						if (length != sizeof(OverlayMsgInit))
-							break;
-
-						OverlayMsgInit *omi = & omMsg.omi;
-
-						uiWidth = omi->uiWidth;
-						uiHeight = omi->uiHeight;
-						qrLast = QRect();
-
-						delete smMem;
-
-						smMem = new SharedMemory2(this, uiWidth * uiHeight * 4);
-						if (! smMem->data()) {
-							qWarning() << "OverlayClient: Failed to create shared memory" << uiWidth << uiHeight;
-							delete smMem;
-							smMem = NULL;
-							break;
-						}
-						QByteArray key = smMem->name().toUtf8();
-						key.append(static_cast<char>(0));
-
-						OverlayMsg om;
-						om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
-						om.omh.uiType = OVERLAY_MSGTYPE_SHMEM;
-						om.omh.iLength = key.length();
-						memcpy(om.oms.a_cName, key.constData(), key.length());
-						qlsSocket->write(om.headerbuffer, sizeof(OverlayMsgHeader) + om.omh.iLength);
-
-						setupRender();
-
-						Overlay *o = static_cast<Overlay *>(parent());
-						QTimer::singleShot(0, o, SLOT(updateOverlay()));
+						readyReadMsgInit(static_cast<unsigned int>(length));
 					}
 					break;
 				case OVERLAY_MSGTYPE_SHMEM: {
@@ -442,7 +459,7 @@ void OverlayClient::readyRead() {
 					}
 					break;
 				case OVERLAY_MSGTYPE_PID: {
-						if (length != sizeof(OverlayMsgPid))
+						if (length != static_cast<qint64>(sizeof(OverlayMsgPid)))
 							break;
 
 						OverlayMsgPid *omp = & omMsg.omp;
@@ -450,13 +467,14 @@ void OverlayClient::readyRead() {
 #ifdef Q_OS_WIN
 						HANDLE h = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, (DWORD)uiPid);
 						if (h) {
-							TCHAR buf[MAX_PATH];
-							if (GetModuleFileNameEx(h, 0, buf, MAX_PATH)) {
-								static_assert(sizeof(TCHAR) == sizeof(wchar_t), "Unicode build required");
-								qsProcessName = QString::fromWCharArray((const wchar_t *) buf);
+							wchar_t buf[MAX_PATH];
+							if (GetModuleFileNameEx(h, 0, buf, MAX_PATH) != 0) {
+								qsExecutablePath = QString::fromWCharArray(buf);
 							}
 							CloseHandle(h);
 						}
+#else
+						qsExecutablePath = QLatin1String("Unknown");
 #endif
 					}
 					break;
@@ -465,7 +483,7 @@ void OverlayClient::readyRead() {
 							break;
 
 						OverlayMsgFps *omf = & omMsg.omf;
-						fFps = omf ->fps;
+						framesPerSecond = omf->fps;
 						//qWarning() << "FPS: " << omf->fps;
 
 						Overlay *o = static_cast<Overlay *>(parent());
@@ -513,7 +531,7 @@ void OverlayClient::setupScene(bool show) {
 			qgpiLogo->setPixmap(QPixmap::fromImage(qir.read()));
 
 			QRectF qrf = qgpiLogo->boundingRect();
-			qgpiLogo->setPos(iroundf((uiWidth - qrf.width()) / 2.0f + 0.5f), iroundf((uiHeight - qrf.height()) / 2.0f) + 0.5f);
+			qgpiLogo->setPos(iroundf((uiWidth - qrf.width()) / 2.0f + 0.5f), iroundf((uiHeight - qrf.height()) / 2.0f + 0.5f));
 
 		}
 
@@ -538,6 +556,7 @@ void OverlayClient::setupScene(bool show) {
 	}
 	ougUsers.updateUsers();
 	updateFPS();
+	updateTime();
 }
 
 void OverlayClient::setupRender() {
@@ -568,6 +587,7 @@ bool OverlayClient::update() {
 
 	ougUsers.updateUsers();
 	updateFPS();
+	updateTime();
 
 	if (qlsSocket->bytesToWrite() > 1024) {
 		return (t.elapsed() <= 5000000ULL);
