@@ -1,41 +1,19 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "mumble_pch.hpp"
 
-#include "Overlay.h"
+#include "OverlayUserGroup.h"
+
+#include "OverlayUser.h"
+#include "OverlayClient.h"
+#include "OverlayEditor.h"
 #include "OverlayText.h"
 #include "User.h"
 #include "Channel.h"
 #include "ClientUser.h"
-#include "Global.h"
 #include "Message.h"
 #include "Database.h"
 #include "NetworkConfig.h"
@@ -43,6 +21,9 @@
 #include "MainWindow.h"
 #include "GlobalShortcut.h"
 
+// We define a global macro called 'g'. This can lead to issues when included code uses 'g' as a type or parameter name (like protobuf 3.7 does). As such, for now, we have to make this our last include.
+#include "Global.h"
+
 template <typename T>
 QRectF OverlayGroup::boundingRect() const {
 	QRectF qr;
@@ -80,18 +61,18 @@ int OverlayUserGroup::type() const {
 	return Type;
 }
 
-void OverlayUserGroup::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) {
-	event->accept();
+void OverlayUserGroup::contextMenuEvent(QGraphicsSceneContextMenuEvent *e) {
+	e->accept();
 
 #ifdef Q_OS_MAC
 	bool embed = g.ocIntercept != NULL;
-	QMenu qm(embed ? NULL : event->widget());
+	QMenu qm(embed ? NULL : e->widget());
 	if (embed) {
 		QGraphicsScene *scene = g.ocIntercept->qgv.scene();
 		scene->addWidget(&qm);
 	}
 #else
-	QMenu qm(g.ocIntercept ? g.mw : event->widget());
+	QMenu qm(g.ocIntercept ? g.mw : e->widget());
 #endif
 
 	QMenu *qmShow = qm.addMenu(OverlayClient::tr("Filter"));
@@ -152,7 +133,7 @@ void OverlayUserGroup::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) {
 	QAction *qaEdit = qm.addAction(OverlayClient::tr("Edit..."));
 	QAction *qaZoom = qm.addAction(OverlayClient::tr("Reset Zoom"));
 
-	QAction *act = qm.exec(event->screenPos());
+	QAction *act = qm.exec(e->screenPos());
 
 	if (! act)
 		return;
@@ -214,26 +195,26 @@ void OverlayUserGroup::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) {
 	}
 }
 
-void OverlayUserGroup::wheelEvent(QGraphicsSceneWheelEvent *event) {
-	event->accept();
+void OverlayUserGroup::wheelEvent(QGraphicsSceneWheelEvent *e) {
+	e->accept();
 
-	qreal scale = 0.875f;
+	qreal scaleFactor = 0.875f;
 
-	if (event->delta() > 0)
-		scale = 1.0f / 0.875f;
+	if (e->delta() > 0)
+		scaleFactor = 1.0f / 0.875f;
 
-	if ((scale < 1.0f) && (os->fZoom <= (1.0f / 4.0f)))
+	if ((scaleFactor < 1.0f) && (os->fZoom <= (1.0f / 4.0f)))
 		return;
-	else if ((scale > 1.0f) && (os->fZoom >= 4.0f))
+	else if ((scaleFactor > 1.0f) && (os->fZoom >= 4.0f))
 		return;
 
-	os->fZoom *= scale;
+	os->fZoom *= scaleFactor;
 
 	updateLayout();
 }
 
-bool OverlayUserGroup::sceneEventFilter(QGraphicsItem *watched, QEvent *event) {
-	switch (event->type()) {
+bool OverlayUserGroup::sceneEventFilter(QGraphicsItem *watched, QEvent *e) {
+	switch (e->type()) {
 		case QEvent::GraphicsSceneMouseMove:
 		case QEvent::GraphicsSceneMouseRelease:
 			QMetaObject::invokeMethod(this, "moveUsers", Qt::QueuedConnection);
@@ -242,7 +223,7 @@ bool OverlayUserGroup::sceneEventFilter(QGraphicsItem *watched, QEvent *event) {
 			break;
 
 	}
-	return OverlayGroup::sceneEventFilter(watched, event);
+	return OverlayGroup::sceneEventFilter(watched, e);
 }
 
 void OverlayUserGroup::moveUsers() {
@@ -252,8 +233,8 @@ void OverlayUserGroup::moveUsers() {
 	const QRectF &sr = scene()->sceneRect();
 	const QPointF &p = qgeiHandle->pos();
 
-	os->fX = qBound<qreal>(0.0f, p.x() / sr.width(), 1.0f);
-	os->fY = qBound<qreal>(0.0f, p.y() / sr.height(), 1.0f);
+	os->fX = static_cast<float>(qBound(0.0, p.x() / sr.width(), 1.0));
+	os->fY = static_cast<float>(qBound(0.0, p.y() / sr.height(), 1.0));
 
 	qgeiHandle->setPos(os->fX * sr.width(), os->fY * sr.height());
 	updateUsers();
@@ -358,31 +339,31 @@ void OverlayUserGroup::updateUsers() {
 		qgi->hide();
 	}
 
-	QRectF children = os->qrfAvatar | os->qrfChannel | os->qrfMutedDeafened | os->qrfUserName;
+	QRectF childrenBounds = os->qrfAvatar | os->qrfChannel | os->qrfMutedDeafened | os->qrfUserName;
 
 	int pad = os->bBox ? iroundf(uiHeight * os->fZoom * (os->fBoxPad + os->fBoxPenWidth) + 0.5f) : 0;
-	int width = iroundf(children.width() * uiHeight * os->fZoom + 0.5f) + 2 * pad;
-	int height = iroundf(children.height() * uiHeight * os->fZoom + 0.5f) + 2 * pad;
+	int width = iroundf(childrenBounds.width() * uiHeight * os->fZoom + 0.5f) + 2 * pad;
+	int height = iroundf(childrenBounds.height() * uiHeight * os->fZoom + 0.5f) + 2 * pad;
 
-	int xofs = - iroundf(children.left() * uiHeight * os->fZoom + 0.5f) + pad;
-	int yofs = - iroundf(children.top() * uiHeight * os->fZoom + 0.5f) + pad;
+	int xOffset = - iroundf(childrenBounds.left() * uiHeight * os->fZoom + 0.5f) + pad;
+	int yOffset = - iroundf(childrenBounds.top() * uiHeight * os->fZoom + 0.5f) + pad;
 
-	unsigned int y = 0;
-	unsigned int x = 0;
+	unsigned int yPos = 0;
+	unsigned int xPos = 0;
 
 	foreach(OverlayUser *ou, users) {
 		if (ou->parentItem() == NULL)
 			ou->setParentItem(this);
 
-		ou->setPos(x * (width+4) + xofs, y * (height + 4) + yofs);
+		ou->setPos(xPos * (width+4) + xOffset, yPos * (height + 4) + yOffset);
 		ou->updateUser();
 		ou->show();
 
-		if (x >= (os->uiColumns - 1)) {
-			x = 0;
-			++y;
+		if (xPos >= (os->uiColumns - 1)) {
+			xPos = 0;
+			++yPos;
 		} else {
-			++x;
+			++xPos;
 		}
 	}
 
