@@ -1,63 +1,45 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "mumble_pch.hpp"
 
 #include "VersionCheck.h"
 
-#include "Global.h"
 #include "MainWindow.h"
 #include "WebFetch.h"
 
+// We define a global macro called 'g'. This can lead to issues when included code uses 'g' as a type or parameter name (like protobuf 3.7 does). As such, for now, we have to make this our last include.
+#include "Global.h"
+
 VersionCheck::VersionCheck(bool autocheck, QObject *p, bool focus) : QObject(p) {
-	bSilent = autocheck;
-
 	QUrl url;
-	url.setPath(focus ? QLatin1String("/focus.php") : QLatin1String("/ver.php"));
+	url.setPath(focus ? QLatin1String("/v1/banner") : QLatin1String("/v1/version-check"));
 
-	url.addQueryItem(QLatin1String("ver"), QLatin1String(QUrl::toPercentEncoding(QLatin1String(MUMBLE_RELEASE))));
-	url.addQueryItem(QLatin1String("date"), QLatin1String(QUrl::toPercentEncoding(QLatin1String(__DATE__))));
-	url.addQueryItem(QLatin1String("time"), QLatin1String(QUrl::toPercentEncoding(QLatin1String(__TIME__))));
+	QList<QPair<QString, QString> > queryItems;
+	queryItems << qMakePair(QString::fromLatin1("ver"), QString::fromLatin1(QUrl::toPercentEncoding(QLatin1String(MUMBLE_RELEASE))));
 #if defined(Q_OS_WIN)
-	url.addQueryItem(QLatin1String("os"), QLatin1String("Win32"));
+# if defined(Q_OS_WIN64)
+	queryItems << qMakePair(QString::fromLatin1("os"), QString::fromLatin1("WinX64"));
+# else
+	queryItems << qMakePair(QString::fromLatin1("os"), QString::fromLatin1("Win32"));
+# endif
 #elif defined(Q_OS_MAC)
-	url.addQueryItem(QLatin1String("os"), QLatin1String("MacOSX"));
+# if defined(USE_MAC_UNIVERSAL)
+	queryItems << qMakePair(QString::fromLatin1("os"), QString::fromLatin1("MacOSX-Universal"));
+# else
+	queryItems << qMakePair(QString::fromLatin1("os"), QString::fromLatin1("MacOSX"));
+# endif
 #else
-	url.addQueryItem(QLatin1String("os"), QLatin1String("Unix"));
+	queryItems << qMakePair(QString::fromLatin1("os"), QString::fromLatin1("Unix"));
 #endif
 	if (! g.s.bUsage)
-		url.addQueryItem(QLatin1String("nousage"), QLatin1String("1"));
+		queryItems << qMakePair(QString::fromLatin1("nousage"), QString::fromLatin1("1"));
 	if (autocheck)
-		url.addQueryItem(QLatin1String("auto"), QLatin1String("1"));
+		queryItems << qMakePair(QString::fromLatin1("auto"), QString::fromLatin1("1"));
 
-	url.addQueryItem(QLatin1String("locale"), g.s.qsLanguage.isEmpty() ? QLocale::system().name() : g.s.qsLanguage);
+	queryItems << qMakePair(QString::fromLatin1("locale"), g.s.qsLanguage.isEmpty() ? QLocale::system().name() : g.s.qsLanguage);
 
 	QFile f(qApp->applicationFilePath());
 	if (! f.open(QIODevice::ReadOnly)) {
@@ -69,20 +51,30 @@ VersionCheck::VersionCheck(bool autocheck, QObject *p, bool focus) : QObject(p)
 		} else {
 			QCryptographicHash qch(QCryptographicHash::Sha1);
 			qch.addData(a);
-			url.addQueryItem(QLatin1String("sha1"), QLatin1String(qch.result().toHex()));
+			queryItems << qMakePair(QString::fromLatin1("sha1"), QString::fromLatin1(qch.result().toHex()));
 		}
 	}
 
-	WebFetch::fetch(url, this, SLOT(fetched(QByteArray,QUrl)));
+#if QT_VERSION >= 0x050000
+	QUrlQuery query;
+	query.setQueryItems(queryItems);
+	url.setQuery(query);
+#else
+	for (int i = 0; i < queryItems.size(); i++) {
+		const QPair<QString, QString> &queryPair = queryItems.at(i);
+		url.addQueryItem(queryPair.first, queryPair.second);
+	}
+#endif
+	WebFetch::fetch(QLatin1String("update"), url, this, SLOT(fetched(QByteArray,QUrl)));
 }
 
 void VersionCheck::fetched(QByteArray a, QUrl url) {
 	if (! a.isNull()) {
 		if (! a.isEmpty()) {
 #ifdef SNAPSHOT_BUILD
-			if (url.path() == QLatin1String("/focus.php")) {
+			if (url.path() == QLatin1String("/v1/banner")) {
 				g.mw->msgBox(QString::fromUtf8(a));
-			} else if (url.path() == QLatin1String("/ver.php")) {
+			} else if (url.path() == QLatin1String("/v1/version-check")) {
 #ifndef Q_OS_WIN
 				g.mw->msgBox(QString::fromUtf8(a));
 #else
@@ -116,7 +108,7 @@ void VersionCheck::fetched(QByteArray a, QUrl url) {
 						data.fdwRevocationChecks = WTD_REVOKE_NONE;
 						data.dwUnionChoice = WTD_CHOICE_FILE;
 						data.pFile = &file;
-						data.dwProvFlags = WTD_SAFER_FLAG | WTD_USE_DEFAULT_OSVER_CHECK | WTD_LIFETIME_SIGNING_FLAG;
+						data.dwProvFlags = WTD_SAFER_FLAG | WTD_USE_DEFAULT_OSVER_CHECK;
 						data.dwUIContext = WTD_UICONTEXT_INSTALL;
 
 						static GUID guid = WINTRUST_ACTION_GENERIC_VERIFY_V2;
@@ -170,7 +162,7 @@ void VersionCheck::fetched(QByteArray a, QUrl url) {
 						}
 					} else {
 						g.mw->msgBox(tr("Downloading new snapshot from %1 to %2").arg(Qt::escape(fetch.toString()), Qt::escape(filename)));
-						WebFetch::fetch(fetch, this, SLOT(fetched(QByteArray,QUrl)));
+						WebFetch::fetch(QLatin1String("dl"), fetch, this, SLOT(fetched(QByteArray,QUrl)));
 						return;
 					}
 				}
@@ -192,7 +184,7 @@ void VersionCheck::fetched(QByteArray a, QUrl url) {
 			g.mw->msgBox(QString::fromUtf8(a));
 #endif
 		}
-	} else if (bSilent) {
+	} else {
 		g.mw->msgBox(tr("Mumble failed to retrieve version information from the central server."));
 	}
 
