@@ -1,22 +1,21 @@
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
+
 #ifndef Q_MOC_RUN
-#ifndef _MUMBLE_PCH_H
-#define _MUMBLE_PCH_H
+#ifndef MUMBLE_MUMBLE_MUMBLE_PCH_H_
+#define MUMBLE_MUMBLE_MUMBLE_PCH_H_
 
-#define QT_NO_DEBUG_STREAM
 #define QT_NO_CAST_TO_ASCII
 #define QT_NO_CAST_FROM_ASCII
 #define QT_USE_FAST_CONCATENATION
 #define QT_USE_FAST_OPERATOR_PLUS
 
 #define NOMINMAX
-#define _WINSOCKAPI_
+#define WIN32_LEAN_AND_MEAN
 
-#if defined(__INTEL_COMPILER)
-#define _MSC_EXTENSIONS
-#include <mathimf.h>
-#else
 #define BOOST_TYPEOF_SUPPRESS_UNNAMED_NAMESPACE
-#endif
 
 #ifdef __APPLE__
 #include <Carbon/Carbon.h>
@@ -27,8 +26,23 @@
 #undef TYPE_BOOL
 #endif
 
+#ifdef __MINGW32__
+// Our MinGW build targets Windows 7 for now.
+// Set up the appropriate Windows macros such that
+// MinGW's Windows headers expose all the functionality
+// we need.
+#define _WIN32_WINNT 0x0601
+#define NTDDI_VERSION NTDDI_WIN7
+#include <ws2tcpip.h>
+#endif
+
 #include <QtCore/QtCore>
 #include <QtGui/QtGui>
+#if QT_VERSION >= 0x050000
+# include "Qt4Compat.h"
+# include <QtWidgets/QtWidgets>
+#endif
+
 #include <QtSvg/QtSvg>
 #ifdef USE_DBUS
 #include <QtDBus/QtDBus>
@@ -38,13 +52,9 @@
 #include <QtXml/QtXml>
 
 #ifdef Q_OS_WIN
-#include <windows.h>
 #define ENABLE_SNDFILE_WINDOWS_PROTOTYPES 1
 #endif
-
-#define __int64_t __int64
 #include <sndfile.h>
-#undef __int64_t
 #include <celt.h>
 #ifdef USE_SBCELT
 #include <sbcelt.h>
@@ -71,42 +81,40 @@
 #include <boost/typeof/typeof.hpp>
 #include <boost/weak_ptr.hpp>
 
+#include <algorithm>
+#include <cmath>
+
+#include "Utils.h"
+
 #ifdef Q_OS_WIN
-#include <shellapi.h>
+#include "../qos2_mingw.h"
+
 #include <winsock2.h>
 #include <qos2.h>
+#include <windows.h>
+#include <shellapi.h>
 #include <wintrust.h>
-#include <Softpub.h>
-#include <Dbt.h>
+#include <softpub.h>
+#include <dbt.h>
 #include <delayimp.h>
 #include <shlobj.h>
 #include <tlhelp32.h>
 #include <psapi.h>
-#ifndef Q_CC_INTEL
 #include <math.h>
-#define lroundf(x) ( static_cast<long int>( (x) + ((x) >= 0.0f ? 0.5f : -0.5f) ) )
-#define lround(x) ( static_cast<long int>( (x) + ((x) >= 0.0 ? 0.5 : -0.5) ) )
-#endif
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif
-#define STACKVAR(type, varname, count) type *varname=reinterpret_cast<type *>(_alloca(sizeof(type) * (count)))
-#else
+#include <mmreg.h>
+#else // ifndef Q_OS_WIN
 #include <math.h>
-#define STACKVAR(type, varname, count) type varname[count]
-#define CopyMemory(dst,ptr,len) memcpy(dst,ptr,len)
-#define ZeroMemory(ptr,len) memset(ptr, 0, len)
-#define __cdecl
-typedef WId HWND;
+#include <sys/socket.h>
+#include <sys/types.h>
 #include <arpa/inet.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
 #endif
 
-#if defined(__MMX__) || defined(Q_OS_WIN)
+#if !defined(Q_OS_OPENBSD) && (defined(__MMX__) || defined(Q_OS_WIN))
 #include <mmintrin.h>
 #endif
 
-#define iroundf(x) ( static_cast<int>(x) )
-
 #ifdef USE_BONJOUR
 #include <dns_sd.h>
 #endif
