@@ -1,32 +1,7 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "murmur_pch.h"
 
@@ -38,20 +13,6 @@
 #include "ServerUser.h"
 #include "Version.h"
 
-/*!
-  \fn void Server::setTempGroups(int userid, Channel *cChannel, const QStringList &groups)
-  Sets the list of temporary groups the given userid is a member of. If no channel is given root will
-  be targeted.
-
-  If userid is negative the absolute value is a session id. If it is positive it is a registration id.
-*/
-
-/*!
-  \fn void Server::clearTempGroups(User *user, Channel *cChannel = NULL, bool recurse = true)
-  Clears temporary group memberships for the given User. If no channel is given root will be targeted.
-  If recursion is activated all temporary memberships in related channels will also be cleared.
-*/
-
 void Server::setUserState(User *pUser, Channel *cChannel, bool mute, bool deaf, bool suppressed, bool prioritySpeaker, const QString& name, const QString &comment) {
 	bool changed = false;
 
@@ -92,9 +53,13 @@ void Server::setUserState(User *pUser, Channel *cChannel, bool mute, bool deaf,
 		mpus.set_name(u8(name));
 	}
 
-	pUser->bDeaf = deaf;
-	pUser->bMute = mute;
-	pUser->bSuppress = suppressed;
+	{
+		QWriteLocker wl(&qrwlVoiceThread);
+		pUser->bDeaf = deaf;
+		pUser->bMute = mute;
+		pUser->bSuppress = suppressed;
+	}
+
 	pUser->bPrioritySpeaker = prioritySpeaker;
 	pUser->qsName = name;
 	hashAssign(pUser->qsComment, pUser->qbaCommentHash, comment);
@@ -117,6 +82,136 @@ void Server::setUserState(User *pUser, Channel *cChannel, bool mute, bool deaf,
 	}
 }
 
+// Sets err to error message on failure.
+bool Server::setChannelStateGRPC(const MumbleProto::ChannelState &cs, QString &err) {
+	::MumbleProto::ChannelState mpcs;
+	bool changed = false;
+	bool updated = false;
+
+	if (!cs.has_channel_id()) {
+		err = QLatin1String("missing channel ID");
+		return false;
+	}
+	Channel *channel = qhChannels.value(cs.channel_id());
+	if (!channel) {
+		err = QLatin1String("invalid channel");
+		return false;
+	}
+	mpcs.set_channel_id(cs.channel_id());
+
+	// Links and parent channel are processed first, because they can return
+	// errors. Without doing this, the server state can be changed without
+	// notifying users.
+	QSet< ::Channel *> newLinksSet;
+	for (int i = 0; i < cs.links_size(); i++) {
+		Channel *link = qhChannels.value(cs.links(i));
+		if (!link) {
+			err = QLatin1String("invalid channel link");
+			return false;
+		}
+		newLinksSet.insert(link);
+	}
+
+	if (cs.has_parent()) {
+		Channel *parent = qhChannels.value(cs.parent());
+		if (!parent) {
+			err = QLatin1String("invalid parent channel");
+			return false;
+		}
+		if (parent != channel->cParent) {
+			Channel *p = parent;
+			while (p) {
+				if (p == channel) {
+					err = QLatin1String("parent channel cannot be a descendant of channel");
+					return false;
+				}
+				p = p->cParent;
+			}
+			if (!canNest(parent, channel)) {
+				err = QLatin1String("channel cannot be nested in the given parent");
+				return false;
+			}
+
+			{
+				QWriteLocker wl(&qrwlVoiceThread);
+				channel->cParent->removeChannel(channel);
+				parent->addChannel(channel);
+			}
+
+			mpcs.set_parent(parent->iId);
+
+			changed = true;
+			updated = true;
+		}
+	}
+
+	if (cs.has_name()) {
+		QString qsName = u8(cs.name());
+		if (channel->qsName != qsName) {
+			channel->qsName = qsName;
+			mpcs.set_name(cs.name());
+
+			changed = true;
+			updated = true;
+		}
+	}
+
+	const QSet<Channel *> &oldLinksSet = channel->qsPermLinks;
+
+	if (newLinksSet != oldLinksSet) {
+		// Remove
+		foreach(Channel *l, oldLinksSet) {
+			if (!newLinksSet.contains(l)) {
+				removeLink(channel, l);
+				mpcs.add_links_remove(l->iId);
+			}
+		}
+		// Add
+		foreach(Channel *l, newLinksSet) {
+			if (! oldLinksSet.contains(l)) {
+				addLink(channel, l);
+				mpcs.add_links_add(l->iId);
+			}
+		}
+
+		changed = true;
+	}
+
+	if (cs.has_position() && cs.position() != channel->iPosition) {
+		channel->iPosition = cs.position();
+		mpcs.set_position(cs.position());
+
+		changed = true;
+		updated = true;
+	}
+
+	if (cs.has_description()) {
+		QString qsDescription = u8(cs.description());
+		if (qsDescription != channel->qsDesc) {
+			hashAssign(channel->qsDesc, channel->qbaDescHash, qsDescription);
+			mpcs.set_description(cs.description());
+
+			changed = true;
+			updated = true;
+		}
+	}
+
+	if (updated) {
+		updateChannel(channel);
+	}
+	if (changed) {
+		sendAll(mpcs, ~ 0x010202);
+		if (mpcs.has_description() && !channel->qbaDescHash.isEmpty()) {
+			mpcs.clear_description();
+			mpcs.set_description_hash(blob(channel->qbaDescHash));
+		}
+		sendAll(mpcs, 0x010202);
+		emit channelStateChanged(channel);
+	}
+
+	return true;
+}
+
 bool Server::setChannelState(Channel *cChannel, Channel *cParent, const QString &qsName, const QSet<Channel *> &links, const QString &desc, const int position) {
 	bool changed = false;
 	bool updated = false;
@@ -143,8 +238,11 @@ bool Server::setChannelState(Channel *cChannel, Channel *cParent, const QString
 			return false;
 		}
 
-		cChannel->cParent->removeChannel(cChannel);
-		cParent->addChannel(cChannel);
+		{
+			QWriteLocker wl(&qrwlVoiceThread);
+			cChannel->cParent->removeChannel(cChannel);
+			cParent->addChannel(cChannel);
+		}
 
 		mpcs.set_parent(cParent->iId);
 
@@ -203,6 +301,67 @@ bool Server::setChannelState(Channel *cChannel, Channel *cParent, const QString
 	return true;
 }
 
+void Server::sendTextMessageGRPC(const ::MumbleProto::TextMessage &tm) {
+	MumbleProto::TextMessage mptm;
+	mptm.set_message(tm.message());
+
+	if (tm.has_actor()) {
+		mptm.set_actor(tm.actor());
+	}
+
+	// Broadcast
+	if (!tm.session_size() && !tm.channel_id_size() && !tm.tree_id_size()) {
+		sendAll(mptm);
+		return;
+	}
+
+	// Single targets
+	for (int i = 0; i < tm.session_size(); i++) {
+		ServerUser *user = qhUsers.value(tm.session(i));
+		if (!user) {
+			continue;
+		}
+		mptm.add_session(user->uiSession);
+		sendMessage(user, mptm);
+		mptm.clear_session();
+	}
+
+	// Channel targets
+	QSet<Channel *> chans;
+
+	for (int i = 0; i < tm.channel_id_size(); i++) {
+		Channel *channel = qhChannels.value(tm.channel_id(i));
+		if (!channel) {
+			continue;
+		}
+		chans.insert(channel);
+		mptm.add_channel_id(channel->iId);
+	}
+
+	QQueue<Channel *> chansQ;
+	for (int i = 0; i < tm.tree_id_size(); i++) {
+		Channel *channel = qhChannels.value(tm.tree_id(i));
+		if (!channel) {
+			continue;
+		}
+		chansQ.enqueue(channel);
+		mptm.add_tree_id(channel->iId);
+	}
+	while (!chansQ.isEmpty()) {
+		Channel *c = chansQ.dequeue();
+		chans.insert(c);
+		foreach(c, c->qlChannels) {
+			chansQ.enqueue(c);
+		}
+	}
+
+	foreach(Channel *c, chans) {
+		foreach(::User *p, c->qlUsers) {
+			sendMessage(static_cast< ::ServerUser *>(p), mptm);
+		}
+	}
+}
+
 void Server::sendTextMessage(Channel *cChannel, ServerUser *pUser, bool tree, const QString &text) {
 	MumbleProto::TextMessage mptm;
 	mptm.set_message(u8(text));
@@ -237,26 +396,36 @@ void Server::sendTextMessage(Channel *cChannel, ServerUser *pUser, bool tree, co
 	}
 }
 
+/**
+ * Sets the list of temporary groups the given userid is a member of. If no channel is given root will
+ * be targeted.
+ *
+ * If userid is negative the absolute value is a session id. If it is positive it is a registration id.
+ */
 void Server::setTempGroups(int userid, int sessionId, Channel *cChannel, const QStringList &groups) {
 	if (! cChannel)
 		cChannel = qhChannels.value(0);
 
-	Group *g;
-	foreach(g, cChannel->qhGroups) {
-		g->qsTemporary.remove(userid);
-		if (sessionId != 0)
-			g->qsTemporary.remove(- sessionId);
-	}
+	{
+		QWriteLocker wl(&qrwlVoiceThread);
 
-	QString gname;
-	foreach(gname, groups) {
-		g = cChannel->qhGroups.value(gname);
-		if (! g) {
-			g = new Group(cChannel, gname);
+		Group *g;
+		foreach(g, cChannel->qhGroups) {
+			g->qsTemporary.remove(userid);
+			if (sessionId != 0)
+				g->qsTemporary.remove(- sessionId);
+		}
+
+		QString gname;
+		foreach(gname, groups) {
+			g = cChannel->qhGroups.value(gname);
+			if (! g) {
+				g = new Group(cChannel, gname);
+			}
+			g->qsTemporary.insert(userid);
+			if (sessionId != 0)
+				g->qsTemporary.insert(- sessionId);
 		}
-		g->qsTemporary.insert(userid);
-		if (sessionId != 0)
-			g->qsTemporary.insert(- sessionId);
 	}
 
 	User *p = qhUsers.value(userid);
@@ -264,6 +433,10 @@ void Server::setTempGroups(int userid, int sessionId, Channel *cChannel, const Q
 		clearACLCache(p);
 }
 
+/**
+ * Clears temporary group memberships for the given User. If no channel is given root will be targeted.
+ * If recursion is activated all temporary memberships in related channels will also be cleared.
+ */
 void Server::clearTempGroups(User *user, Channel *cChannel, bool recurse) {
 	QList<Channel*> qlChans;
 	if (! cChannel)
@@ -271,16 +444,20 @@ void Server::clearTempGroups(User *user, Channel *cChannel, bool recurse) {
 
 	qlChans.append(cChannel);
 
-	while (!qlChans.isEmpty()) {
-		Channel *chan = qlChans.takeLast();
-		Group *g;
-		foreach(g, chan->qhGroups) {
-			g->qsTemporary.remove(user->iId);
-			g->qsTemporary.remove(- static_cast<int>(user->uiSession));
-		}
+	{
+		QWriteLocker wl(&qrwlVoiceThread);
 
-		if (recurse)
-			qlChans << chan->qlChannels;
+		while (!qlChans.isEmpty()) {
+			Channel *chan = qlChans.takeLast();
+			Group *g;
+			foreach(g, chan->qhGroups) {
+				g->qsTemporary.remove(user->iId);
+				g->qsTemporary.remove(-static_cast<int>(user->uiSession));
+			}
+
+			if (recurse)
+				qlChans << chan->qlChannels;
+		}
 	}
 
 	clearACLCache(user);
