@@ -1,33 +1,7 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "murmur_pch.h"
 
@@ -43,6 +17,10 @@
 #include "PacketDataStream.h"
 #include "ServerDB.h"
 #include "ServerUser.h"
+#include "Version.h"
+#include "HTMLFilter.h"
+#include "HostAddress.h"
+#include "EnvUtils.h"
 
 #ifdef USE_BONJOUR
 #include "BonjourServer.h"
@@ -55,13 +33,6 @@
 
 #define UDP_PACKET_SIZE 1024
 
-LogEmitter::LogEmitter(QObject *p) : QObject(p) {
-};
-
-void LogEmitter::addLogEntry(const QString &msg) {
-	emit newLogEntry(msg);
-};
-
 ExecEvent::ExecEvent(boost::function<void ()> f) : QEvent(static_cast<QEvent::Type>(EXEC_QEVENT)) {
 	func = f;
 }
@@ -73,7 +44,47 @@ void ExecEvent::execute() {
 SslServer::SslServer(QObject *p) : QTcpServer(p) {
 }
 
+bool SslServer::hasDualStackSupport() {
+	// Create a AF_INET6 socket and try to switch off IPV6_V6ONLY. This
+	// should only fail if the system does not support dual-stack mode
+	// for this socket type.
+
+	bool result = false;
+#ifdef Q_OS_UNIX
+	int s = ::socket(AF_INET6, SOCK_STREAM, 0);
+	if (s != -1) {
+		const int ipv6only = 0;
+		if (setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, reinterpret_cast<const char*>(&ipv6only), sizeof(ipv6only)) == 0) {
+			result = true;
+		}
+		::close(s);
+	}
+#else
+	WSADATA wsaData;
+	WORD wVersionRequested = MAKEWORD(2, 2);
+	if (WSAStartup(wVersionRequested, &wsaData) != 0) {
+		// Seems like we won't be doing any network stuff anyways
+		return false;
+	}
+
+	SOCKET s = ::WSASocket(AF_INET6, SOCK_STREAM, IPPROTO_TCP, NULL, 0, WSA_FLAG_OVERLAPPED);
+	if (s != INVALID_SOCKET) {
+		const int ipv6only = 0;
+		if (setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, reinterpret_cast<const char*>(&ipv6only), sizeof(ipv6only)) == 0) {
+			result = true;
+		}
+		closesocket(s);
+	}
+	WSACleanup();
+#endif
+	return result;
+}
+
+#if QT_VERSION >= 0x050000
+void SslServer::incomingConnection(qintptr v) {
+#else
 void SslServer::incomingConnection(int v) {
+#endif
 	QSslSocket *s = new QSslSocket(this);
 	s->setSocketDescriptor(v);
 	qlSockets.append(s);
@@ -91,6 +102,7 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 #ifdef USE_BONJOUR
 	bsRegistration = NULL;
 #endif
+	bUsingMetaCert = false;
 
 #ifdef Q_OS_UNIX
 	aiNotify[0] = aiNotify[1] = -1;
@@ -128,7 +140,7 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 	foreach(SslServer *ss, qlServer) {
 		sockaddr_storage addr;
 #ifdef Q_OS_UNIX
-		int tcpsock = ss->socketDescriptor();
+		int tcpsock = static_cast<int>(ss->socketDescriptor());
 		socklen_t len = sizeof(addr);
 #else
 		SOCKET tcpsock = ss->socketDescriptor();
@@ -162,6 +174,20 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 			bValid = false;
 			return;
 		} else {
+			if (addr.ss_family == AF_INET6) {
+				// Copy IPV6_V6ONLY attribute from tcp socket, it defaults to nonzero on Windows
+				// See https://msdn.microsoft.com/en-us/library/windows/desktop/ms738574%28v=vs.85%29.aspx
+				// This will fail for WindowsXP which is ok. Our TCP code will have split that up
+				// into two sockets.
+				int ipv6only = 0;
+				socklen_t optlen = sizeof(ipv6only);
+				if (::getsockopt(tcpsock, IPPROTO_IPV6, IPV6_V6ONLY, reinterpret_cast<char*>(&ipv6only), &optlen) == 0) {
+					if (::setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, reinterpret_cast<const char*>(&ipv6only), optlen) == SOCKET_ERROR) {
+						log(QString("Failed to copy IPV6_V6ONLY socket attribute from tcp to udp socket"));
+					}
+				}
+			}
+
 			if (::bind(sock, reinterpret_cast<sockaddr *>(&addr), len) == SOCKET_ERROR) {
 				log(QString("Failed to bind UDP Socket to %1").arg(addressToString(ss->serverAddress(), usPort)));
 			} else {
@@ -307,6 +333,41 @@ Server::~Server() {
 	log("Stopped");
 }
 
+/// normalizeSuggestVersion normalizes a 'suggestversion' config value.
+/// The config value may be a version string, or a bitmasked
+/// integer representing the version.
+/// This function converts the 'suggestversion' config value to
+/// always be a bitmasked integer representation.
+///
+/// On error, the function returns an empty QVariant.
+static QVariant normalizeSuggestVersion(QVariant suggestVersion) {
+	uint integerValue = suggestVersion.toUInt();
+
+	// If the integer value is 0, it can mean two things:
+	//
+	// Either the suggestversion is set to 0.
+	// Or, the suggestversion is a version string such as "1.3.0",
+	// and cannot be converted to an integer value.
+	//
+	// We handle both cases the same: by pretending the
+	// suggestversion is a version string in both cases.
+	//
+	// If it is a version string, the call to MumbleVersion::getRaw()
+	// will return the bitmasked representation.
+	//
+	// If it is not a version string, the call to MumbleVersion::getRaw()
+	// will return 0, so it is effectively a no-op.
+	if (integerValue == 0) {
+		integerValue = MumbleVersion::getRaw(suggestVersion.toString());
+	}
+
+	if (integerValue != 0) {
+		return integerValue;
+	}
+
+	return QVariant();
+}
+
 void Server::readParams() {
 	qsPassword = Meta::mp.qsPassword;
 	usPort = static_cast<unsigned short>(Meta::mp.usPort + iServerNum - 1);
@@ -329,13 +390,17 @@ void Server::readParams() {
 	bBonjour = Meta::mp.bBonjour;
 	bAllowPing = Meta::mp.bAllowPing;
 	bCertRequired = Meta::mp.bCertRequired;
+	bForceExternalAuth = Meta::mp.bForceExternalAuth;
 	qrUserName = Meta::mp.qrUserName;
 	qrChannelName = Meta::mp.qrChannelName;
+	iMessageLimit = Meta::mp.iMessageLimit;
+	iMessageBurst = Meta::mp.iMessageBurst;
 	qvSuggestVersion = Meta::mp.qvSuggestVersion;
 	qvSuggestPositional = Meta::mp.qvSuggestPositional;
 	qvSuggestPushToTalk = Meta::mp.qvSuggestPushToTalk;
 	iOpusThreshold = Meta::mp.iOpusThreshold;
 	iChannelNestingLimit = Meta::mp.iChannelNestingLimit;
+	iChannelCountLimit = Meta::mp.iChannelCountLimit;
 
 	QString qsHost = getConf("host", QString()).toString();
 	if (! qsHost.isEmpty()) {
@@ -385,8 +450,9 @@ void Server::readParams() {
 	bBonjour = getConf("bonjour", bBonjour).toBool();
 	bAllowPing = getConf("allowping", bAllowPing).toBool();
 	bCertRequired = getConf("certrequired", bCertRequired).toBool();
+	bForceExternalAuth = getConf("forceExternalAuth", bForceExternalAuth).toBool();
 
-	qvSuggestVersion = getConf("suggestversion", qvSuggestVersion);
+	qvSuggestVersion = normalizeSuggestVersion(getConf("suggestversion", qvSuggestVersion));
 	if (qvSuggestVersion.toUInt() == 0)
 		qvSuggestVersion = QVariant();
 
@@ -401,9 +467,19 @@ void Server::readParams() {
 	iOpusThreshold = getConf("opusthreshold", iOpusThreshold).toInt();
 
 	iChannelNestingLimit = getConf("channelnestinglimit", iChannelNestingLimit).toInt();
+	iChannelCountLimit = getConf("channelcountlimit", iChannelCountLimit).toInt();
 
 	qrUserName=QRegExp(getConf("username", qrUserName.pattern()).toString());
 	qrChannelName=QRegExp(getConf("channelname", qrChannelName.pattern()).toString());
+
+	iMessageLimit=getConf("messagelimit", iMessageLimit).toUInt();
+	if (iMessageLimit < 1) { // Prevent disabling messages entirely
+		iMessageLimit = 1;
+	}
+	iMessageBurst=getConf("messageburst", iMessageBurst).toUInt();
+	if (iMessageBurst < 1) { // Prevent disabling messages entirely
+		iMessageBurst = 1;
+	}
 }
 
 void Server::setLiveConf(const QString &key, const QString &value) {
@@ -431,6 +507,10 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 		for (int id = 1; id < iMaxUsers * 2; ++id)
 			if (!qhUsers.contains(id))
 				qqIds.enqueue(id);
+
+		MumbleProto::ServerConfig mpsc;
+		mpsc.set_max_users(iMaxUsers);
+		sendAll(mpsc);
 	} else if (key == "usersperchannel")
 		iMaxUsersPerChannel = i ? i : Meta::mp.iMaxUsersPerChannel;
 	else if (key == "textmessagelength") {
@@ -465,11 +545,9 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 		QString text = !v.isNull() ? v : Meta::mp.qsWelcomeText;
 		if (text != qsWelcomeText) {
 			qsWelcomeText = text;
-			if (! qsWelcomeText.isEmpty()) {
-				MumbleProto::ServerConfig mpsc;
-				mpsc.set_welcome_text(u8(qsWelcomeText));
-				sendAll(mpsc);
-			}
+			MumbleProto::ServerConfig mpsc;
+			mpsc.set_welcome_text(u8(qsWelcomeText));
+			sendAll(mpsc);
 		}
 	} else if (key == "registername") {
 		QString text = !v.isNull() ? v : Meta::mp.qsRegName;
@@ -492,6 +570,8 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 		qurlRegWeb = !v.isNull() ? v : Meta::mp.qurlRegWeb;
 	else if (key == "certrequired")
 		bCertRequired = !v.isNull() ? QVariant(v).toBool() : Meta::mp.bCertRequired;
+	else if (key == "forceExternalAuth")
+		bForceExternalAuth = !v.isNull() ? QVariant(v).toBool() : Meta::mp.bForceExternalAuth;
 	else if (key == "bonjour") {
 		bBonjour = !v.isNull() ? QVariant(v).toBool() : Meta::mp.bBonjour;
 #ifdef USE_BONJOUR
@@ -507,15 +587,28 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 	else if (key == "channelname")
 		qrChannelName=!v.isNull() ? QRegExp(v) : Meta::mp.qrChannelName;
 	else if (key == "suggestversion")
-		qvSuggestVersion = ! v.isNull() ? (v.isEmpty() ? QVariant() : v) : Meta::mp.qvSuggestVersion;
+		qvSuggestVersion = ! v.isNull() ? (v.isEmpty() ? QVariant() : normalizeSuggestVersion(v)) : Meta::mp.qvSuggestVersion;
 	else if (key == "suggestpositional")
 		qvSuggestPositional = ! v.isNull() ? (v.isEmpty() ? QVariant() : v) : Meta::mp.qvSuggestPositional;
 	else if (key == "suggestpushtotalk")
 		qvSuggestPushToTalk = ! v.isNull() ? (v.isEmpty() ? QVariant() : v) : Meta::mp.qvSuggestPushToTalk;
 	else if (key == "opusthreshold")
 		iOpusThreshold = (i >= 0 && !v.isNull()) ? qBound(0, i, 100) : Meta::mp.iOpusThreshold;
-	else if (key =="channelnestinglimit")
+	else if (key == "channelnestinglimit")
 		iChannelNestingLimit = (i >= 0 && !v.isNull()) ? i : Meta::mp.iChannelNestingLimit;
+	else if (key == "channelcountlimit")
+		iChannelCountLimit = (i >= 0 && !v.isNull()) ? i : Meta::mp.iChannelCountLimit;
+	else if (key == "messagelimit") {
+		iMessageLimit = (!v.isNull()) ? v.toUInt() : Meta::mp.iMessageLimit;
+		if (iMessageLimit < 1) {
+			iMessageLimit = 1;
+		}
+	} else if (key == "messageburst") {
+		iMessageBurst = (!v.isNull()) ? v.toUInt() : Meta::mp.iMessageBurst;
+		if (iMessageBurst < 1) {
+			iMessageBurst = 1;
+		}
+	}
 }
 
 #ifdef USE_BONJOUR
@@ -700,6 +793,7 @@ void Server::run() {
 				msg.msg_controllen = sizeof(controldata);
 
 				len=static_cast<quint32>(::recvmsg(sock, &msg, MSG_TRUNC));
+				Q_UNUSED(fromlen);
 #else
 				len=static_cast<qint32>(::recvfrom(sock, encrypt, UDP_PACKET_SIZE, MSG_TRUNC, reinterpret_cast<struct sockaddr *>(&from), &fromlen));
 #endif
@@ -715,7 +809,7 @@ void Server::run() {
 					continue;
 				}
 
-				QReadLocker rl(&qrwlUsers);
+				QReadLocker rl(&qrwlVoiceThread);
 
 				quint32 *ping = reinterpret_cast<quint32 *>(encrypt);
 
@@ -749,23 +843,23 @@ void Server::run() {
 				} else {
 					// Unknown peer
 					foreach(ServerUser *usr, qhHostUsers.value(ha)) {
-						if (usr->csCrypt.isValid() && checkDecrypt(usr, encrypt, buffer, len)) {
+						if (checkDecrypt(usr, encrypt, buffer, len)) { // checkDecrypt takes the User's qrwlCrypt lock.
 							// Every time we relock, reverify users' existance.
 							// The main thread might delete the user while the lock isn't held.
 							unsigned int uiSession = usr->uiSession;
 							rl.unlock();
-							qrwlUsers.lockForWrite();
+							qrwlVoiceThread.lockForWrite();
 							if (qhUsers.contains(uiSession)) {
 								u = usr;
 								u->sUdpSocket = sock;
 								memcpy(& u->saiUdpAddress, &from, sizeof(from));
 								qhHostUsers[from].remove(u);
 								qhPeerUsers.insert(key, u);
-								qrwlUsers.unlock();
-								rl.relock();
-								if (! qhUsers.contains(uiSession))
-									u = NULL;
 							}
+							qrwlVoiceThread.unlock();
+							rl.relock();
+							if (u != NULL && !qhUsers.contains(uiSession))
+								u = NULL;
 							break;
 						}
 					}
@@ -777,21 +871,25 @@ void Server::run() {
 
 				MessageHandler::UDPMessageType msgType = static_cast<MessageHandler::UDPMessageType>((buffer[0] >> 5) & 0x7);
 
-				switch (msgType) {
-					case MessageHandler::UDPVoiceSpeex:
-					case MessageHandler::UDPVoiceCELTAlpha:
-					case MessageHandler::UDPVoiceCELTBeta:
-						if (bOpus)
-							break;
-					case MessageHandler::UDPVoiceOpus: {
-							u->bUdp = true;
-							processMsg(u, buffer, len);
-							break;
-						}
-					case MessageHandler::UDPPing: {
-							QByteArray qba;
-							sendMessage(u, buffer, len, qba, true);
-						}
+				if (msgType == MessageHandler::UDPVoiceSpeex ||
+				    msgType == MessageHandler::UDPVoiceCELTAlpha ||
+				    msgType == MessageHandler::UDPVoiceCELTBeta ||
+				    msgType == MessageHandler::UDPVoiceOpus) {
+
+					// Allow all voice packets through by default.
+					bool ok = true;
+					// ...Unless we're in Opus mode. In Opus mode, only Opus packets are allowed.
+					if (bOpus && msgType != MessageHandler::UDPVoiceOpus) {
+						ok = false;
+					}
+
+					if (ok) {
+						u->aiUdpFlag = 1;
+						processMsg(u, buffer, len);
+					}
+				} else if (msgType == MessageHandler::UDPPing) {
+					QByteArray qba;
+					sendMessage(u, buffer, len, qba, true);
 				}
 #ifdef Q_OS_UNIX
 				fds[i].revents = 0;
@@ -808,6 +906,8 @@ void Server::run() {
 }
 
 bool Server::checkDecrypt(ServerUser *u, const char *encrypt, char *plain, unsigned int len) {
+	QMutexLocker l(&u->qmCrypt);
+
 	if (u->csCrypt.isValid() && u->csCrypt.decrypt(reinterpret_cast<const unsigned char *>(encrypt), reinterpret_cast<unsigned char *>(plain), len))
 		return true;
 
@@ -821,18 +921,28 @@ bool Server::checkDecrypt(ServerUser *u, const char *encrypt, char *plain, unsig
 }
 
 void Server::sendMessage(ServerUser *u, const char *data, int len, QByteArray &cache, bool force) {
-	if ((u->bUdp || force) && (u->sUdpSocket != INVALID_SOCKET) && u->csCrypt.isValid()) {
+	if ((QAtomicIntLoad(u->aiUdpFlag) == 1 || force) && (u->sUdpSocket != INVALID_SOCKET)) {
 #if defined(__LP64__)
 		STACKVAR(char, ebuffer, len+4+16);
 		char *buffer = reinterpret_cast<char *>(((reinterpret_cast<quint64>(ebuffer) + 8) & ~7) + 4);
 #else
 		STACKVAR(char, buffer, len+4);
 #endif
-		u->csCrypt.encrypt(reinterpret_cast<const unsigned char *>(data), reinterpret_cast<unsigned char *>(buffer), len);
+		{
+			QMutexLocker wl(&u->qmCrypt);
+
+			if (!u->csCrypt.isValid()) {
+				return;
+			}
+
+			if (!u->csCrypt.encrypt(reinterpret_cast<const unsigned char *>(data), reinterpret_cast<unsigned char *>(buffer), len)) {
+				return;
+			}
+		}
 #ifdef Q_OS_WIN
 		DWORD dwFlow = 0;
 		if (Meta::hQoS)
-			QOSAddSocketToFlow(Meta::hQoS, u->sUdpSocket, reinterpret_cast<struct sockaddr *>(& u->saiUdpAddress), QOSTrafficTypeVoice, QOS_NON_ADAPTIVE_FLOW, &dwFlow);
+			QOSAddSocketToFlow(Meta::hQoS, u->sUdpSocket, reinterpret_cast<struct sockaddr *>(& u->saiUdpAddress), QOSTrafficTypeVoice, QOS_NON_ADAPTIVE_FLOW, reinterpret_cast<PQOS_FLOWID>(&dwFlow));
 #endif
 #ifdef Q_OS_LINUX
 		struct msghdr msg;
@@ -846,7 +956,7 @@ void Server::sendMessage(ServerUser *u, const char *data, int len, QByteArray &c
 
 		memset(&msg, 0, sizeof(msg));
 		msg.msg_name = reinterpret_cast<struct sockaddr *>(& u->saiUdpAddress);
-		msg.msg_namelen = (u->saiUdpAddress.ss_family == AF_INET6) ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in);
+		msg.msg_namelen = static_cast<socklen_t>((u->saiUdpAddress.ss_family == AF_INET6) ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in));
 		msg.msg_iov = iov;
 		msg.msg_iovlen = 1;
 		msg.msg_control = controldata;
@@ -901,9 +1011,6 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 	if (u->sState != ServerUser::Authenticated || u->bMute || u->bSuppress || u->bSelfMute)
 		return;
 
-	User *p;
-	BandwidthRecord *bw = & u->bwr;
-	Channel *c = u->cChannel;
 	QByteArray qba, qba_npos;
 	unsigned int counter;
 	char buffer[UDP_PACKET_SIZE];
@@ -913,13 +1020,17 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 	unsigned int target = data[0] & 0x1f;
 	unsigned int poslen;
 
-	// IP + UDP + Crypt + Data
-	int packetsize = 20 + 8 + 4 + len;
-
 	// Check the voice data rate limit.
-	if (! bw->addFrame(packetsize, iMaxBandwidth/8)) {
-		// Suppress packet.
-		return;
+	{
+		BandwidthRecord *bw = &u->bwr;
+
+		// IP + UDP + Crypt + Data
+		const int packetsize = 20 + 8 + 4 + len;
+
+		if (! bw->addFrame(packetsize, iMaxBandwidth / 8)) {
+			// Suppress packet.
+			 return;
+		}
 	}
 
 	// Read the sequence number.
@@ -947,13 +1058,23 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 
 	len = pds.size() + 1;
 
+	if (poslen > static_cast<unsigned int>(len)) {
+		// poslen must never ever be bigger than len as this could lead to negative buffer sizes (len - poslen) being
+		// used when further processing the packet.
+		// Usually this shouldn't happen in the first place but can happen with malformed/malicious packets in certain
+		// cases.
+		poslen = 0;
+	}
+
 	if (target == 0x1f) { // Server loopback
 		buffer[0] = static_cast<char>(type | 0);
 		sendMessage(u, buffer, len, qba);
 		return;
 	} else if (target == 0) { // Normal speech
+		Channel *c = u->cChannel;
+
 		buffer[0] = static_cast<char>(type | 0);
-		foreach(p, c->qlUsers) {
+		foreach(User *p, c->qlUsers) {
 			ServerUser *pDst = static_cast<ServerUser *>(p);
 			SENDTO;
 		}
@@ -966,7 +1087,7 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 
 			foreach(Channel *l, chans) {
 				if (ChanACL::hasPermission(u, l, ChanACL::Speak, &acCache)) {
-					foreach(p, l->qlUsers) {
+					foreach(User *p, l->qlUsers) {
 						ServerUser *pDst = static_cast<ServerUser *>(p);
 						SENDTO;
 					}
@@ -995,7 +1116,7 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 						if (!link && !dochildren && ! group) {
 							// Common case
 							if (ChanACL::hasPermission(u, wc, ChanACL::Whisper, &acCache)) {
-								foreach(p, wc->qlUsers) {
+								foreach(User *p, wc->qlUsers) {
 									channel.insert(static_cast<ServerUser *>(p));
 								}
 							}
@@ -1011,7 +1132,7 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 							const QString &qsg = redirect.isEmpty() ? wtc.qsGroup : redirect;
 							foreach(Channel *tc, channels) {
 								if (ChanACL::hasPermission(u, tc, ChanACL::Whisper, &acCache)) {
-									foreach(p, tc->qlUsers) {
+									foreach(User *p, tc->qlUsers) {
 										ServerUser *su = static_cast<ServerUser *>(p);
 										if (! group || Group::isMember(tc, tc, qsg, su)) {
 											channel.insert(su);
@@ -1024,20 +1145,24 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 				}
 			}
 
-			foreach(unsigned int id, wt.qlSessions) {
-				ServerUser *pDst = qhUsers.value(id);
-				if (pDst && ChanACL::hasPermission(u, pDst->cChannel, ChanACL::Whisper, &acCache) && ! channel.contains(pDst))
-					direct.insert(pDst);
+			{
+				QMutexLocker qml(&qmCache);
+
+				foreach(unsigned int id, wt.qlSessions) {
+					ServerUser *pDst = qhUsers.value(id);
+					if (pDst && ChanACL::hasPermission(u, pDst->cChannel, ChanACL::Whisper, &acCache) && !channel.contains(pDst))
+						direct.insert(pDst);
+				}
 			}
 
 			int uiSession = u->uiSession;
-			qrwlUsers.unlock();
-			qrwlUsers.lockForWrite();
+			qrwlVoiceThread.unlock();
+			qrwlVoiceThread.lockForWrite();
 
 			if (qhUsers.contains(uiSession))
 				u->qmTargetCache.insert(target, ServerUser::TargetCache(channel, direct));
-			qrwlUsers.unlock();
-			qrwlUsers.lockForRead();
+			qrwlVoiceThread.unlock();
+			qrwlVoiceThread.lockForRead();
 			if (! qhUsers.contains(uiSession))
 				return;
 		}
@@ -1105,17 +1230,49 @@ void Server::newClient() {
 
 		foreach(const Ban &ban, qlBans) {
 			if (ban.haAddress.match(ha, ban.iMask)) {
-				log(QString("Ignoring connection: %1 (Server ban)").arg(addressToString(sock->peerAddress(), sock->peerPort())));
+				log(QString("Ignoring connection: %1, Reason: %2, Username: %3, Hash: %4 (Server ban)").arg(addressToString(sock->peerAddress(), sock->peerPort()), ban.qsReason, ban.qsUsername, ban.qsHash));
 				sock->disconnectFromHost();
 				sock->deleteLater();
 				return;
 			}
 		}
 
+#ifdef Q_OS_MAC
+		// One unexpected behavior of Qt's SSL backend is: it will add the key pair
+		// it uses in a connection into the default keychain, and when access the private
+		// key afterwards, a pop up will show up asking for user's permission.
+		// In some case (OS X 10.15.5), this pop up will be suppressed somehow and no private
+		// key is returned.
+		// This env variable will avoid Qt directly adding the key pair into the default keychain,
+		// using a temporary keychain instead.
+		// See #4298 and https://codereview.qt-project.org/c/qt/qtbase/+/184243
+		EnvUtils::setenv("QT_SSL_USE_TEMPORARY_KEYCHAIN", "1");
+#endif
 		sock->setPrivateKey(qskKey);
 		sock->setLocalCertificate(qscCert);
+
+		// Treat the leaf certificate as a root.
+		// This shouldn't strictly be necessary,
+		// and is a left-over from early on.
+		// Perhaps it is necessary for self-signed
+		// certs?
 		sock->addCaCertificate(qscCert);
-		sock->addCaCertificates(qlCA);
+
+		// Add CA certificates specified via
+		// murmur.ini's sslCA option.
+		sock->addCaCertificates(Meta::mp.qlCA);
+
+		// Add intermediate CAs found in the PEM
+		// bundle used for this server's certificate.
+		sock->addCaCertificates(qlIntermediates);
+
+		sock->setCiphers(Meta::mp.qlCiphers);
+
+#if defined(USE_QSSLDIFFIEHELLMANPARAMETERS)
+		QSslConfiguration cfg = sock->sslConfiguration();
+		cfg.setDiffieHellmanParameters(qsdhpDHParams);
+		sock->setSslConfiguration(cfg);
+#endif
 
 		if (qqIds.isEmpty()) {
 			log(QString("Session ID pool (%1) empty, rejecting connection").arg(iMaxUsers));
@@ -1130,7 +1287,7 @@ void Server::newClient() {
 		HostAddress(sock->localAddress()).toSockaddr(& u->saiTcpLocalAddress);
 
 		{
-			QWriteLocker wl(&qrwlUsers);
+			QWriteLocker wl(&qrwlVoiceThread);
 			qhUsers.insert(u->uiSession, u);
 			qhHostUsers[ha].insert(u);
 		}
@@ -1144,7 +1301,17 @@ void Server::newClient() {
 
 		u->setToS();
 
+#if QT_VERSION >= 0x050500
+		sock->setProtocol(QSsl::TlsV1_0OrLater);
+#elif QT_VERSION >= 0x050400
+		// In Qt 5.4, QSsl::SecureProtocols is equivalent
+		// to "TLSv1.0 or later", which we require.
+		sock->setProtocol(QSsl::SecureProtocols);
+#elif QT_VERSION >= 0x050000
+		sock->setProtocol(QSsl::TlsV1_0);
+#else
 		sock->setProtocol(QSsl::TlsV1);
+#endif
 		sock->startServerEncryption();
 	}
 }
@@ -1168,15 +1335,36 @@ void Server::encrypted() {
 	QList<QSslCertificate> certs = uSource->peerCertificateChain();
 	if (!certs.isEmpty()) {
 		const QSslCertificate &cert = certs.last();
+#if QT_VERSION >= 0x050000
+		uSource->qslEmail = cert.subjectAlternativeNames().values(QSsl::EmailEntry);
+#else
 		uSource->qslEmail = cert.alternateSubjectNames().values(QSsl::EmailEntry);
+#endif
 		uSource->qsHash = cert.digest(QCryptographicHash::Sha1).toHex();
 		if (! uSource->qslEmail.isEmpty() && uSource->bVerified) {
-			log(uSource, QString("Strong certificate for %1 <%2> (signed by %3)").arg(cert.subjectInfo(QSslCertificate::CommonName)).arg(uSource->qslEmail.join(", ")).arg(certs.first().issuerInfo(QSslCertificate::CommonName)));
+#if QT_VERSION >= 0x050000
+			QString subject;
+			QString issuer;
+
+			QStringList subjectList = cert.subjectInfo(QSslCertificate::CommonName);
+			if (! subjectList.isEmpty()) {
+				subject = subjectList.first();
+			}
+
+			QStringList issuerList = certs.first().issuerInfo(QSslCertificate::CommonName);
+			if (! issuerList.isEmpty()) {
+				issuer = issuerList.first();
+			}
+#else
+			QString subject = cert.subjectInfo(QSslCertificate::CommonName);
+			QString issuer = certs.first().issuerInfo(QSslCertificate::CommonName);
+#endif
+			log(uSource, QString::fromUtf8("Strong certificate for %1 <%2> (signed by %3)").arg(subject).arg(uSource->qslEmail.join(", ")).arg(issuer));
 		}
 
 		foreach(const Ban &ban, qlBans) {
 			if (ban.qsHash == uSource->qsHash) {
-				log(uSource, QString("Certificate hash is banned."));
+				log(uSource, QString("Certificate hash is banned: %1, Username: %2, Reason: %3.").arg(ban.qsHash, ban.qsUsername, ban.qsReason));
 				uSource->disconnectSocket();
 			}
 		}
@@ -1209,13 +1397,63 @@ void Server::sslError(const QList<QSslError> &errors) {
 		}
 	}
 
-	if (ok)
+	if (ok) {
 		u->proceedAnyway();
-	else
+	} else {
+		// Due to a regression in Qt 5 (QTBUG-53906),
+		// we can't 'force' disconnect (which calls
+		// QAbstractSocket->abort()) when built against Qt 5.
+		//
+		// The bug is that Qt doesn't update the
+		// QSslSocket's socket state when QSslSocket->abort()
+		// is called.
+		//
+		// Our call to abort() happens when QSslSocket is inside
+		// startHandshake(). That is, a handshake is in progress.
+		//
+		// After emitting the peerVerifyError/sslErrors signals,
+		// startHandshake() checks whether the connection is still
+		// in QAbstractSocket::ConectedState.
+		//
+		// Unfortunately, because abort() doesn't update the socket's
+		// state to signal that it is no longer connected, startHandshake()
+		// still thinks the socket is connected and will continue to
+		// attempt to finish the handshake.
+		//
+		// Because abort() tears down a lot of internal state
+		// of the QSslSocket, inlcuding the 'SSL *' object
+		// associated with the socket, this is fatal and leads
+		// to crashes, such as attempting to derefernce a NULL
+		// 'SSL *' object.
+		//
+		// To avoid this, we use a non-forceful disconnect
+		// until this is fixed upstream.
+		//
+		// See
+		// https://bugreports.qt.io/browse/QTBUG-53906
+		// https://github.com/mumble-voip/mumble/issues/2334
+#if QT_VERSION >= 0x050000
+		u->disconnectSocket();
+#else
 		u->disconnectSocket(true);
+#endif
+	}
 }
 
 void Server::connectionClosed(QAbstractSocket::SocketError err, const QString &reason) {
+	if (reason.contains(QLatin1String("140E0197"))) {
+		// A severe bug was introduced in qt/qtbase@93a803a6de27d9eb57931c431b5f3d074914f693.
+		// q_SSL_shutdown() causes Qt to emit "error()" from unrelated QSslSocket(s), in addition to the correct one.
+		// The issue causes this function to disconnect random authenticated clients.
+		//
+		// The workaround consists in ignoring a specific OpenSSL error:
+		// "Error while reading: error:140E0197:SSL routines:SSL_shutdown:shutdown while in init [20]"
+		//
+		// Definitely not ideal, but it fixes a critical vulnerability.
+		qWarning("Ignored OpenSSL error 140E0197 for %p", sender());
+		return;
+	}
+
 	Connection *c = qobject_cast<Connection *>(sender());
 	if (! c)
 		return;
@@ -1238,7 +1476,7 @@ void Server::connectionClosed(QAbstractSocket::SocketError err, const QString &r
 	Channel *old = u->cChannel;
 
 	{
-		QWriteLocker wl(&qrwlUsers);
+		QWriteLocker wl(&qrwlVoiceThread);
 
 		qhUsers.remove(u->uiSession);
 		qhHostUsers[u->haAddress].remove(u);
@@ -1278,29 +1516,35 @@ void Server::message(unsigned int uiType, const QByteArray &qbaMsg, ServerUser *
 	}
 
 	if (uiType == MessageHandler::UDPTunnel) {
-		int l = qbaMsg.size();
-		if (l < 2)
+		int len = qbaMsg.size();
+		if (len < 2 || len > UDP_PACKET_SIZE) {
+			// Drop messages that are too small to be senseful or that are bigger than allowed
 			return;
+		}
 
-		QReadLocker rl(&qrwlUsers);
+		QReadLocker rl(&qrwlVoiceThread);
 
-		u->bUdp = false;
+		u->aiUdpFlag = 0;
 
 		const char *buffer = qbaMsg.constData();
 
 		MessageHandler::UDPMessageType msgType = static_cast<MessageHandler::UDPMessageType>((buffer[0] >> 5) & 0x7);
 
-		switch (msgType) {
-			case MessageHandler::UDPVoiceCELTAlpha:
-			case MessageHandler::UDPVoiceCELTBeta:
-			case MessageHandler::UDPVoiceSpeex:
-				if (bOpus)
-					break;
-			case MessageHandler::UDPVoiceOpus:
-				processMsg(u, buffer, l);
-				break;
-			default:
-				break;
+		if (msgType == MessageHandler::UDPVoiceSpeex ||
+		    msgType == MessageHandler::UDPVoiceCELTAlpha ||
+		    msgType == MessageHandler::UDPVoiceCELTBeta ||
+		    msgType == MessageHandler::UDPVoiceOpus) {
+
+			// Allow all voice packets through by default.
+			bool ok = true;
+			// ...Unless we're in Opus mode. In Opus mode, only Opus packets are allowed.
+			if (bOpus && msgType != MessageHandler::UDPVoiceOpus) {
+				ok = false;
+			}
+
+			if (ok) {
+				processMsg(u, buffer, len);
+			}
 		}
 
 		return;
@@ -1338,14 +1582,14 @@ void Server::message(unsigned int uiType, const QByteArray &qbaMsg, ServerUser *
 void Server::checkTimeout() {
 	QList<ServerUser *> qlClose;
 
-	qrwlUsers.lockForRead();
+	qrwlVoiceThread.lockForRead();
 	foreach(ServerUser *u, qhUsers) {
 		if (u->activityTime() > (iTimeout * 1000)) {
 			log(u, "Timeout");
 			qlClose.append(u);
 		}
 	}
-	qrwlUsers.unlock();
+	qrwlVoiceThread.unlock();
 	foreach(ServerUser *u, qlClose)
 		u->disconnectSocket(true);
 }
@@ -1406,17 +1650,23 @@ void Server::removeChannel(Channel *chan, Channel *dest) {
 	if (dest == NULL)
 		dest = chan->cParent;
 
-	chan->unlink(NULL);
+	{
+		QWriteLocker wl(&qrwlVoiceThread);
+		chan->unlink(NULL);
+	}
 
 	foreach(c, chan->qlChannels) {
 		removeChannel(c, dest);
 	}
 
 	foreach(p, chan->qlUsers) {
-		chan->removeUser(p);
+		{
+			QWriteLocker wl(&qrwlVoiceThread);
+			chan->removeUser(p);
+		}
 
 		Channel *target = dest;
-		while (target->cParent && ! hasPermission(static_cast<ServerUser *>(p), target, ChanACL::Enter))
+		while (target->cParent && (! hasPermission(static_cast<ServerUser *>(p), target, ChanACL::Enter) || isChannelFull(target, static_cast<ServerUser *>(p))))
 			target = target->cParent;
 
 		MumbleProto::UserState mpus;
@@ -1435,7 +1685,7 @@ void Server::removeChannel(Channel *chan, Channel *dest) {
 	emit channelRemoved(chan);
 
 	if (chan->cParent) {
-		QWriteLocker wl(&qrwlUsers);
+		QWriteLocker wl(&qrwlVoiceThread);
 		chan->cParent->removeChannel(chan);
 	}
 
@@ -1491,7 +1741,7 @@ void Server::userEnterChannel(User *p, Channel *c, MumbleProto::UserState &mpus)
 	Channel *old = p->cChannel;
 
 	{
-		QWriteLocker wl(&qrwlUsers);
+		QWriteLocker wl(&qrwlVoiceThread);
 		c->addUser(p);
 
 		bool mayspeak = ChanACL::hasPermission(static_cast<ServerUser *>(p), c, ChanACL::Speak, NULL);
@@ -1621,7 +1871,7 @@ void Server::clearACLCache(User *p) {
 	}
 
 	{
-		QWriteLocker lock(&qrwlUsers);
+		QWriteLocker lock(&qrwlVoiceThread);
 
 		foreach(ServerUser *u, qhUsers)
 			u->qmTargetCache.clear();
@@ -1758,29 +2008,11 @@ bool Server::isTextAllowed(QString &text, bool &changed) {
 	changed = false;
 
 	if (! bAllowHTML) {
-		if (! text.contains(QLatin1Char('<'))) {
-			text = text.simplified();
-		} else {
-			QXmlStreamReader qxsr(QString::fromLatin1("<document>%1</document>").arg(text));
-			QString qs;
-			while (! qxsr.atEnd()) {
-				switch (qxsr.readNext()) {
-					case QXmlStreamReader::Invalid:
-						return false;
-					case QXmlStreamReader::Characters:
-						qs += qxsr.text();
-						break;
-					case QXmlStreamReader::EndElement:
-						if ((qxsr.name() == QLatin1String("br")) || (qxsr.name() == QLatin1String("p")))
-							qs += "\n";
-						break;
-					default:
-						break;
-				}
-			}
-			text = qs.simplified();
+		QString out;
+		if (HTMLFilter::filter(text, out)) {
+			changed = true;
+			text = out;
 		}
-		changed = true;
 		return ((iMaxTextMessageLength == 0) || (text.length() <= iMaxTextMessageLength));
 	} else {
 		int length = text.length();
@@ -1801,6 +2033,8 @@ bool Server::isTextAllowed(QString &text, bool &changed) {
 		if (! text.contains(QLatin1Char('<')))
 			return false;
 
+		// Strip value from <img>s src attributes to check text-length only -
+		// we already ensured the img-length requirement is met
 		QString qsOut;
 		QXmlStreamReader qxsr(QString::fromLatin1("<document>%1</document>").arg(text));
 		QXmlStreamWriter qxsw(&qsOut);
@@ -1810,8 +2044,6 @@ bool Server::isTextAllowed(QString &text, bool &changed) {
 					return false;
 				case QXmlStreamReader::StartElement: {
 						if (qxsr.name() == QLatin1String("img")) {
-							QXmlStreamAttributes attr = qxsr.attributes();
-
 							qxsw.writeStartElement(qxsr.namespaceUri().toString(), qxsr.name().toString());
 							foreach(const QXmlStreamAttribute &a, qxsr.attributes())
 								if (a.name() != QLatin1String("src"))
@@ -1833,9 +2065,22 @@ bool Server::isTextAllowed(QString &text, bool &changed) {
 	}
 }
 
+bool Server::isChannelFull(Channel *c, ServerUser *u) {
+	if (u && hasPermission(u, c, ChanACL::Write)) {
+		return false;
+	}
+	if (c->uiMaxUsers) {
+		return static_cast<unsigned int>(c->qlUsers.count()) >= c->uiMaxUsers;
+	}
+	if (iMaxUsersPerChannel) {
+		return c->qlUsers.count() >= iMaxUsersPerChannel;
+	}
+	return false;
+}
+
 bool Server::canNest(Channel *newParent, Channel *channel) const {
-	const int parentLevel = newParent ? newParent->getLevel() : -1;
-	const int channelDepth = channel ? channel->getDepth() : 0;
+	const int parentLevel = newParent ? static_cast<int>(newParent->getLevel()) : -1;
+	const int channelDepth = channel ? static_cast<int>(channel->getDepth()) : 0;
 
 	return (parentLevel + channelDepth) < iChannelNestingLimit;
 }
