@@ -1,38 +1,13 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
-
-#ifndef SERVERUSER_H_
-#define SERVERUSER_H_
+#ifndef MUMBLE_MURMUR_SERVERUSER_H_
+#define MUMBLE_MURMUR_SERVERUSER_H_
 
 #include <QtCore/QStringList>
+#include <QtCore/QElapsedTimer>
 
 #ifdef Q_OS_UNIX
 #include <sys/socket.h>
@@ -41,9 +16,9 @@
 #endif
 
 #include "Connection.h"
-#include "Net.h"
 #include "Timer.h"
 #include "User.h"
+#include "HostAddress.h"
 
 // Unfortunately, this needs to be "large enough" to hold
 // enough frames to account for both short-term and
@@ -58,6 +33,7 @@ struct BandwidthRecord {
 	Timer tIdleControl;
 	unsigned short a_iBW[N_BANDWIDTH_SLOTS];
 	Timer a_qtWhen[N_BANDWIDTH_SLOTS];
+	mutable QMutex qmMutex;
 
 	BandwidthRecord();
 	bool addFrame(int size, int maxpersec);
@@ -80,6 +56,34 @@ struct WhisperTarget {
 
 class Server;
 
+/// A simple implementation for rate-limiting.
+/// See https://en.wikipedia.org/wiki/Leaky_bucket
+class LeakyBucket {
+	private:
+		/// The amount of tokens that are drained per second.
+		/// (The sze of the whole in the bucket)
+		unsigned int m_tokensPerSec;
+		/// The maximum amount of tokens that may be encountered.
+		/// (The capacity of the bucket)
+		unsigned int m_maxTokens;
+		/// The amount of tokens currently stored
+		/// (The amount of whater currently in the bucket)
+		long m_currentTokens;
+		/// A timer that is used to measure time intervals. It is essential
+		/// that this timer uses a monotonic clock (which is why QElapsedTimer is
+		/// used instead of QTime or QDateTime).
+		QElapsedTimer m_timer;
+
+	public:
+		/// @param tokens The amount of tokens that should be added.
+		/// @returns Whether adding this amount of tokens triggers rate
+		/// 	limiting (true means the corresponding packet has to be
+		/// 	discared and false means the packet may be processed)
+		bool ratelimit(int tokens);
+
+		LeakyBucket(unsigned int tokensPerSec, unsigned int maxTokens);
+};
+
 class ServerUser : public Connection, public User {
 	private:
 		Q_OBJECT
@@ -89,11 +93,11 @@ class ServerUser : public Connection, public User {
 	public:
 		enum State { Connected, Authenticated };
 		State sState;
-		operator const QString() const;
+		operator QString() const;
 
 		float dUDPPingAvg, dUDPPingVar;
 		float dTCPPingAvg, dTCPPingVar;
-		quint64 uiUDPPackets, uiTCPPackets;
+		quint32 uiUDPPackets, uiTCPPackets;
 
 		unsigned int uiVersion;
 		QString qsRelease;
@@ -107,7 +111,16 @@ class ServerUser : public Connection, public User {
 		QStringList qslEmail;
 
 		HostAddress haAddress;
-		bool bUdp;
+
+		/// Holds whether the user is using TCP
+		/// or UDP for voice packets.
+		///
+		/// If the flag is 0, the user is using
+		/// TCP.
+		///
+		/// If the flag is 1, the user is using
+		/// UDP.
+		QAtomicInt aiUdpFlag;
 
 		QList<int> qlCodecs;
 		bool bOpus;
@@ -119,6 +132,8 @@ class ServerUser : public Connection, public User {
 		QMap<int, TargetCache> qmTargetCache;
 		QMap<QString, QString> qmWhisperRedirect;
 
+		LeakyBucket leakyBucket;
+
 		int iLastPermissionCheck;
 		QMap<int, unsigned int> qmPermissionSent;
 #ifdef Q_OS_UNIX
