@@ -1,40 +1,17 @@
-/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
-
-   All rights reserved.
-
-   Redistribution and use in source and binary forms, with or without
-   modification, are permitted provided that the following conditions
-   are met:
-
-   - Redistributions of source code must retain the above copyright notice,
-     this list of conditions and the following disclaimer.
-   - Redistributions in binary form must reproduce the above copyright notice,
-     this list of conditions and the following disclaimer in the documentation
-     and/or other materials provided with the distribution.
-   - Neither the name of the Mumble Developers nor the names of its
-     contributors may be used to endorse or promote products derived from this
-     software without specific prior written permission.
-
-   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
-   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
-   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
-   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-*/
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
 
 #include "murmur_pch.h"
 
 #include "Tray.h"
 
+#include "About.h"
 #include "Meta.h"
 #include "Server.h"
 #include "Version.h"
+#include "LogEmitter.h"
 
 Tray::Tray(QObject *p, LogEmitter *logger) : QObject(p) {
 	le = logger;
@@ -47,6 +24,9 @@ Tray::Tray(QObject *p, LogEmitter *logger) : QObject(p) {
 	qaQuit->setShortcut(tr("Ctrl+Q", "Quit"));
 	qaQuit->setObjectName(QLatin1String("Quit"));
 
+	qaAbout = new QAction(tr("&About Murmur"), this);
+	qaAbout->setObjectName(QLatin1String("About"));
+
 	qaShowLog = new QAction(tr("&Show Log"), this);
 	qaShowLog->setShortcut(tr("Ctrl+L", "Quit"));
 	qaShowLog->setObjectName(QLatin1String("ShowLog"));
@@ -56,6 +36,7 @@ Tray::Tray(QObject *p, LogEmitter *logger) : QObject(p) {
 	qm = new QMenu(tr("Murmur"), NULL);
 	qm->addAction(qaShowLog);
 	qm->addSeparator();
+	qm->addAction(qaAbout);
 	qm->addAction(qaQuit);
 	qsti->setContextMenu(qm);
 
@@ -78,6 +59,11 @@ void Tray::on_Quit_triggered() {
 	}
 }
 
+void Tray::on_About_triggered() {
+	AboutDialog ad(NULL);
+	ad.exec();
+}
+
 void Tray::on_ShowLog_triggered() {
 	QMainWindow *mw = new QMainWindow();
 	mw->setAttribute(Qt::WA_DeleteOnClose);
