@@ -1,13 +1,15 @@
+// Copyright 2005-2019 The Mumble Developers. All rights reserved.
+// Use of this source code is governed by a BSD-style license
+// that can be found in the LICENSE file at the root of the
+// Mumble source tree or at <https://www.mumble.info/LICENSE>.
+
 #ifndef Q_MOC_RUN
-#ifndef _MURMUR_PCH_H
-#define _MURMUR_PCH_H
+#ifndef MUMBLE_MURMUR_MURMUR_PCH_H_
+#define MUMBLE_MURMUR_MURMUR_PCH_H_
 
 #define WIN32_LEAN_AND_MEAN
 
 #define _USE_MATH_DEFINES
-#if defined(__INTEL_COMPILER)
-#include <mathimf.h>
-#endif
 
 #ifdef __APPLE__
 #include <CoreFoundation/CoreFoundation.h>
@@ -16,6 +18,12 @@
 #undef TYPE_BOOL
 #endif
 
+#ifdef __MINGW32__
+#define _WIN32_WINNT 0x0600
+#include <ws2tcpip.h>
+#include <mswsock.h>
+#endif
+
 #include <QtCore/QtCore>
 #include <QtNetwork/QtNetwork>
 #include <QtSql/QtSql>
@@ -24,8 +32,17 @@
 #include <QtDBus/QtDBus>
 #endif
 
+#include "QAtomicIntCompat.h"
+
 #ifdef Q_OS_WIN
 #include <QtGui/QtGui>
+#if QT_VERSION >= 0x050000
+# include "Qt4Compat.h"
+# include <QtWidgets/QtWidgets>
+#endif
+
+#include "../qos2_mingw.h"
+
 #include <winsock2.h>
 #include <qos2.h>
 #include <windows.h>
@@ -47,6 +64,7 @@ extern "C" {
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <sys/resource.h>
+#include <syslog.h>
 #ifdef Q_OS_LINUX
 #include <linux/types.h> // needed to work around evil magic stuff in capability.h
 #include <sys/capability.h>
@@ -54,7 +72,7 @@ extern "C" {
 #endif
 #include <pwd.h>
 #include <grp.h>
-#ifdef __FreeBSD__
+#if defined __FreeBSD__ || defined __OpenBSD__
 #include <netinet/in_systm.h>
 #endif
 #include <netinet/in.h>
@@ -62,33 +80,26 @@ extern "C" {
 #include <errno.h>
 #include <signal.h>
 #include <poll.h>
-#define INVALID_SOCKET -1
-#define SOCKET_ERROR -1
 #ifdef Q_OS_DARWIN
 #include <poll.h>
 #endif
 #endif
 
-#if !defined (Q_CC_INTEL) && !defined (Q_OS_WIN)
 #include <math.h>
-#endif
-#if defined (Q_OS_WIN) && (defined (Q_CC_INTEL) || defined (Q_CC_MSVC))
-#define lroundf(x) ( static_cast<int>( (x) + ((x) >= 0 ? 0.5 : -0.5) ) )
-#define snprintf ::_snprintf
-#define STACKVAR(type, varname, count) type *varname=reinterpret_cast<type *>(_alloca(sizeof(type) * (count)))
-#else
-#define STACKVAR(type, varname, count) type varname[count]
-#endif
 
 #include <boost/function.hpp>
 #include <boost/bind.hpp>
 #include <boost/shared_ptr.hpp>
 #include <boost/weak_ptr.hpp>
 
+#include "Utils.h"
+
 #ifdef USE_BONJOUR
 #include <dns_sd.h>
 #endif
 
+#include <openssl/opensslv.h>
+#include <openssl/bn.h>
 #include <openssl/aes.h>
 #include <openssl/rand.h>
 #include <openssl/pem.h>
